/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.interfaz;

import java.awt.BorderLayout;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import uniandes.cupi2.inscripcionCedulas.excepciones.CiudadanoRegistradoException;
import uniandes.cupi2.inscripcionCedulas.excepciones.FormatoArchivoException;
import uniandes.cupi2.inscripcionCedulas.excepciones.PersistenciaException;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.GeneradorReporteHTML;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.GeneradorReportePuestoTXT;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.InterpreteCSV;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.Serializador;
import uniandes.cupi2.inscripcionCedulas.mundo.Departamento;
import uniandes.cupi2.inscripcionCedulas.mundo.RangoEdad;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;
import uniandes.cupi2.inscripcionCedulas.mundo.InscripcionCedulas;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;

/**
 * Ventana principal de la aplicacin.
 */
public class InterfazInscripcionCedulas extends JFrame
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Constante que representa la ruta del archivo donde se guarda la informacin del sistema.
     */
    private final static String DATA = "./data/inscripcionCedulas.data";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase principal del mundo.
     */
    private InscripcionCedulas inscripcionCedulas;

    // -----------------------------------------------------------------
    // Atributos de la interfaz
    // -----------------------------------------------------------------

    /**
     * Panel con las extensiones.
     */
    private PanelExtensiones panelExtension;

    /**
     * Panel con la imagen del encabezado.
     */
    private PanelImagen panelImagen;

    /**
     * Panel con las opciones del usuario.
     */
    private PanelOpciones panelOpciones;

    /**
     * Panel con la informacin del sistemas de inscripcin.
     */
    private PanelInformacion panelInformacion;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Constructor de la ventana de la aplicacin. <br>
     * <b>post: </b> Los paneles de la ventana han sido inicializados y la clase principal del mundo ha sido creada.
     */
    public InterfazInscripcionCedulas( )
    {

    	inscripcionCedulas = new InscripcionCedulas();
    	// TODO 8.A. Trate de deserializar la informacin del archivo DATA y guardarla en inscripcionCedulas
    	//		Si no es posible debe mostrar un mensaje al usuario indicando lo ocurrido


        // Construye la forma.
        setLayout( new BorderLayout( ) );
        setSize( 800, 500 );
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        setTitle( "Sistema de inscripcin de cdulas" );

        // Creacin de los paneles aqu.
        panelImagen = new PanelImagen( );
        add( panelImagen, BorderLayout.NORTH );

        panelExtension = new PanelExtensiones( this );
        add( panelExtension, BorderLayout.SOUTH );

        panelOpciones = new PanelOpciones( this );
        add( panelOpciones, BorderLayout.WEST );

        panelInformacion = new PanelInformacion( this );
        add( panelInformacion, BorderLayout.CENTER );

        setLocationRelativeTo( null );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Retorna los departamentos del sistema de inscripcin de cdulas.
     * @return Lista de departamentos del sistema.
     */
    public List<Departamento> darDepartamentos( )
    {
        return inscripcionCedulas.darDepartamentos( );
    }

    
    /**
     * Retorna el porcentaje de hombres del sistema de inscripcin de cdulas.
     * @return Porcentaje de hombres.
     */
    public double darPorcentajeHombres( )
    {
        return inscripcionCedulas.calcularPorcentajeInscritosGenero(Genero.MASCULINO);
    }

    /**
     * Retorna el nmero de ciudadanos inscritos en el rango dado por parmetro.
     * @param pRango Rango de edad. pRango != null && (pRango == RANGO_1 || pRango == RANGO_2 || pRango == RANGO_3 || pRango == RANGO_4 || pRango == RANGO_5 || pRango ==
     *        RANGO_6 ).
     * @return Ciudadanos inscritos en el rango de edad dado.
     */
    public int darInscritosRangoEdad( RangoEdad pRango )
    {
        return inscripcionCedulas.calcularInscritosRangoEdad( pRango );
    }

    /**
     * Retorna el porcentaje de mujeres del sistemas de inscripcin de cdulas.
     * @return Porcentaje de mujeres.
     */
    public double darPorcentajeMujeres( )
    {
    	return inscripcionCedulas.calcularPorcentajeInscritosGenero(Genero.FEMENINO);
    }
    
    /**
     * Despliega el dialogo que inscribe un ciudadano al sistema.
     */
    public void inscribirCiudadano( )
    {
        PuestoVotacion p = panelInformacion.darPuestoSeleccionado( );

        if( p != null )
        {
            DialogoInscribirCiudadano dialogo = new DialogoInscribirCiudadano( this );
            dialogo.setVisible( true );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "No ha seleccionado un puesto de votacin.", "Inscribir ciudadano", JOptionPane.ERROR_MESSAGE );
        }
    }

    /**
     * Inscribe el ciudadano con la informacin dada por parmetro.
     * @param pCedula Cdula del ciudadano. pCedula != null && pCedula != "".
     * @param pApellidos Apellidos del ciudadano. pApellidos != null && pApellidos != "".
     * @param pNombres Nombres del ciudadano. pNombres != null && pNombres != "".
     * @param pFechaNacimiento Fecha de nacimiento del ciudadano. pFechaNacimiento != null.
     * @param pLugarNacimiento Lugar de nacimiento del ciudadano. pLugarNacimiento != null && pLugarNacimiento != "".
     * @param pSexo Sexo del ciudadano. pSexo != null && (pSexo == FEMENINO || pSexo == MASCULINO ).
     * @param pFechaExpedicion Fecha de expedicin de la cdula del ciudadano. pExpedicion != null.
     * @param pLugarExpedicion Lugar de expedicin de la cdula del ciudadano. pLugarExpedicion != null && pLugarExpedicion != "".
     */
    public void inscribir( String pCedula, String pApellidos, String pNombres, Date pFechaNacimiento, String pLugarNacimiento, Genero pSexo, Date pFechaExpedicion, String pLugarExpedicion )
    {
        PuestoVotacion p = panelInformacion.darPuestoSeleccionado( );

        try
        {
            inscripcionCedulas.inscribirCiudadano(  p, pCedula, pApellidos, pNombres, pFechaNacimiento, pLugarNacimiento, pSexo, pFechaExpedicion, pLugarExpedicion );
            panelInformacion.actualizarListaInscritos( p );
        }
        catch( CiudadanoRegistradoException e )
        {
            JOptionPane.showMessageDialog( this, e.getMessage( ), "Inscribir ciudadano", JOptionPane.ERROR_MESSAGE );
        }
    }

    /**
     * Importa la informacin del sistema de inscripcin de cdulas de un archivo seleccionado por el usuario.<br>
     * <b>post: </b> Se ha importado la informacin del sistema y se ha actualizado la ventana de la aplicacin.
     */
    public void importar( )
    {
        JFileChooser chooser = new JFileChooser( "./data" );
        chooser.setDialogTitle( "Importar datos" );
        int returnVal = chooser.showSaveDialog( this );
        if( returnVal == JFileChooser.APPROVE_OPTION )
        {
            String ruta = chooser.getSelectedFile( ).getPath( );
            try
            {
                InterpreteCSV.importarCSV(ruta, inscripcionCedulas);
                panelInformacion.actualizarDepartamentos( );
                panelOpciones.activarOpciones();
            }
            catch( PersistenciaException e )
            {
                JOptionPane.showMessageDialog( this, e.getMessage( ), "Importar datos", JOptionPane.ERROR_MESSAGE );
            }
            catch( FormatoArchivoException e )
            {
                JOptionPane.showMessageDialog( this, e.getMessage( ), "Importar datos", JOptionPane.ERROR_MESSAGE );
            }
        }

    }

    /**
     * Genera un reporte de los ciudadanos inscritos en el puesto de votacin seleccionado.<br>
     * El usuario debe seleccionar la carpeta destino del reporte.
     */
    public void generarReporte( )
    {
        Departamento d = panelInformacion.darDepartamentoSeleccionado( );
        Municipio m = panelInformacion.darMunicipioSeleccionado( );
        PuestoVotacion p = panelInformacion.darPuestoSeleccionado( );

        if( p != null )
        {
            JFileChooser chooser = new JFileChooser( "./data" );
            chooser.setDialogTitle( "Generar reporte" );
            int returnVal = chooser.showSaveDialog( this );
            if( returnVal == JFileChooser.APPROVE_OPTION )
            {
                String ruta = chooser.getSelectedFile( ).getAbsolutePath();
                try
                {
                    GeneradorReportePuestoTXT.generarReportePuesto(d, m, p, ruta);
                    JOptionPane.showMessageDialog( this, "Reporte de: " + p.darNombre() + " generado.", "Generar reporte", JOptionPane.INFORMATION_MESSAGE );
                }
                catch( PersistenciaException e )
                {
                    JOptionPane.showMessageDialog( this, e.getMessage( ), "Generar reporte", JOptionPane.ERROR_MESSAGE );
                }
            }
        }
        else
        {
            JOptionPane.showMessageDialog( this, "No ha seleccionado un puesto de votacin.", "Generar reporte", JOptionPane.ERROR_MESSAGE );
        }
    }
    
    /**
     * Genera un reporte de los ciudadanos inscritos en los puestos del municipio seleccionado.<br>
     * El usuario debe seleccionar la carpeta destino del reporte.
     */
    public void generarReporteHTML( )
    {
        Municipio m = panelInformacion.darMunicipioSeleccionado( );

        if( m != null )
        {
            JFileChooser chooser = new JFileChooser( "./data" );
            chooser.setDialogTitle( "Generar reporte" );
            int returnVal = chooser.showSaveDialog( this );
            if( returnVal == JFileChooser.APPROVE_OPTION )
            {
                String ruta = chooser.getSelectedFile( ).getParent();
                try
                {
                    GeneradorReporteHTML.generarReporte(m, ruta);
                    JOptionPane.showMessageDialog( this, "Reporte de: " + m.darNombre() + " generado.", "Generar reporte", JOptionPane.INFORMATION_MESSAGE );
                }
                catch( PersistenciaException e )
                {
                    JOptionPane.showMessageDialog( this, e.getMessage( ), "Generar reporte", JOptionPane.ERROR_MESSAGE );
                }
            }
        }
        else
        {
            JOptionPane.showMessageDialog( this, "No ha seleccionado un puesto de votacin.", "Generar reporte", JOptionPane.ERROR_MESSAGE );
        }
    }

    /**
     * Este mtodo se encarga de salvar la informacin del sistema, justo antes de cerrar la aplicacin.
     */
    public void dispose( )
    {
        //TODO 8.B Complete el mtodo que se activa al cerrar la aplicacin
    	//		Se debe serializar el estado de inscripcionCedulas en el archivo DATA
    	//		Si no se puede completar la serializacin se debe preguntar al usuario si quiere cerrar la aplicacin
    	
    }

    // -----------------------------------------------------------------
    // Puntos de Extensin
    // -----------------------------------------------------------------

    /**
     * Mtodo para la extensin 1.
     */
    public void reqFuncOpcion1( )
    {
    	try {
			GeneradorReporteHTML.generarReporte(panelInformacion.darMunicipioSeleccionado(), "data");
		} catch (PersistenciaException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        String resultado = inscripcionCedulas.metodo1( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    /**
     * Mtodo para la extensin 2.
     */
    public void reqFuncOpcion2( )
    {
        String resultado = inscripcionCedulas.metodo2( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    // -----------------------------------------------------------------
    // Main
    // -----------------------------------------------------------------

    /**
     * Este mtodo ejecuta la aplicacin, creando una nueva interfaz.
     * @param pArgs Argumentos para la ejecucin de la aplicacin. En este caso no son necesarios.
     */
    public static void main( String[] pArgs )
    {

        InterfazInscripcionCedulas interfaz = new InterfazInscripcionCedulas( );
        interfaz.setVisible( true );
    }
}