/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.interfaz;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Departamento;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;

/**
 * Panel donde se muestra la informacin de departamentos, municipios, puestos de votacin y ciudadanos inscritos.
 */
public class PanelInformacion extends JPanel implements ActionListener
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Constante que representa el comando para cambio de departamento.
     */
    private static final String CAMBIO_DEPARTAMENTO = "DEPARTAMENTO";

    /**
     * Constante que representa el comando para cambio de municipio.
     */
    private static final String CAMBIO_MUNICIPIO = "MUNICICPIO";

    /**
     * Constante que representa el comando para cambio de puesto de votacin.
     */
    private static final String CAMBIO_PUESTO = "PUESTO";

    /**
     * Constante que representa el comando para ver la informacin del ciudadano.
     */
    private static final String VER_INFO = "Ver informacin";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Ventana principal de la aplicacin.
     */
    private InterfazInscripcionCedulas principal;

    // -----------------------------------------------------------------
    // Atributos de interfaz
    // -----------------------------------------------------------------

    /**
     * ComboBox con los departamentos.
     */ 
    //TODO 5.A Declare el atributo comboDepartamentos.

    /**
     * ComboBox con los municipios.
     */
    private JComboBox comboMunicipios;

    /**
     * ComboBox con los puestos de votacin.
     */
    private JComboBox comboPuestos;

    /**
     * Lista de ciudadanos inscritos.
     */
    private JList listaInscritos;

    /**
     * Scroll de la lista de ciudadanos.
     */
    private JScrollPane scrollInscritos;

    /**
     * Campo de texto con el nombre del puesto de votacin.
     */
    private JTextField txtNombre;

    /**
     * Campo de texto con la direccin del puesto de votacin.
     */
    private JTextField txtDireccion;

    /**
     * Botn para ver la informacin del ciudadano.
     */
    private JButton btnVerInfo;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Constructor del panel.
     * @param pPrincipal Ventana principal de la aplicacin. pPrincipal != null.
     */
    public PanelInformacion( InterfazInscripcionCedulas pPrincipal )
    {
        principal = pPrincipal;

        setLayout( new BorderLayout( ) );

        JPanel aux = new JPanel( );
        aux.setLayout( new GridBagLayout( ) );
        setSize( 800, 200 );
        GridBagConstraints gridBagConstraints = new GridBagConstraints( );
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets( 5, 5, 5, 5 );
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;

        aux.add( new JLabel( " Departamentos " ), gridBagConstraints );
        gridBagConstraints.gridx = 1;
        
        //TODO 5.B Inicialice y configure el atributo comboDeportes. Utilice la constante CAMBIO_DEPARTAMENTO como comando asociado.
       
        aux.add( comboDepartamentos, gridBagConstraints );

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        aux.add( new JLabel( " Municipios " ), gridBagConstraints );
        gridBagConstraints.gridx = 1;
        comboMunicipios = new JComboBox( );
        comboMunicipios.setActionCommand( CAMBIO_MUNICIPIO );
        comboMunicipios.addActionListener( this );
        aux.add( comboMunicipios, gridBagConstraints );

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        aux.add( new JLabel( " Puesto de votacin " ), gridBagConstraints );
        gridBagConstraints.gridx = 1;
        comboPuestos = new JComboBox( );
        comboPuestos.setActionCommand( CAMBIO_PUESTO );
        comboPuestos.addActionListener( this );
        aux.add( comboPuestos, gridBagConstraints );

        JPanel aux2 = new JPanel( );
        aux2.setLayout( new GridBagLayout( ) );

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;

        aux2.setPreferredSize( new Dimension( 0, 120 ) );
        aux2.setBorder( new TitledBorder( "Puesto de votacin" ) );

        aux2.add( new JLabel( " Nombre: " ), gridBagConstraints );
        gridBagConstraints.gridx = 1;
        txtNombre = new JTextField( );
        txtNombre.setEditable( false );
        aux2.add( txtNombre, gridBagConstraints );

        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        aux2.add( new JLabel( " Direccin " ), gridBagConstraints );
        gridBagConstraints.gridx = 1;
        txtDireccion = new JTextField( );
        txtDireccion.setEditable( false );
        aux2.add( txtDireccion, gridBagConstraints );

        JPanel aux3 = new JPanel( );
        aux3.setLayout( new BorderLayout( ) );
        aux3.add( aux, BorderLayout.CENTER );
        aux3.add( aux2, BorderLayout.SOUTH );

        add( aux3, BorderLayout.CENTER );

        JPanel aux4 = new JPanel( );
        aux4.setLayout( new BorderLayout( ) );

        listaInscritos = new JList( );

        scrollInscritos = new JScrollPane( listaInscritos );
        scrollInscritos.setHorizontalScrollBarPolicy( JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        scrollInscritos.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
        scrollInscritos.setBorder( new TitledBorder( "Ciudadanos inscritos" ) );

        scrollInscritos.setSize( new Dimension(300, 0));
        scrollInscritos.setMaximumSize(new Dimension(300, 0));
        scrollInscritos.setPreferredSize(new Dimension(300, 0));
        scrollInscritos.setMaximumSize(new Dimension(300, 0));
        aux4.add( scrollInscritos, BorderLayout.CENTER );

        btnVerInfo = new JButton( VER_INFO );
        btnVerInfo.setActionCommand( VER_INFO );
        btnVerInfo.addActionListener( this );
        aux4.add( btnVerInfo, BorderLayout.SOUTH );

        add( aux4, BorderLayout.EAST );
        actualizarDepartamentos( );
        asignarEstilo();
    }
    
    private void asignarEstilo()
    {
    	Color c = new Color(0, 152, 255);
    	btnVerInfo.setBackground(c);
    	btnVerInfo.setForeground(Color.WHITE);
    	btnVerInfo.setBorder(BorderFactory.createLineBorder(Color.WHITE));
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Retorna el departamento seleccionado actualmente.
     * @return Departamento seleccionado.
     */
    public Departamento darDepartamentoSeleccionado( )
    {
    	//TODO 5.C. Devuelve el departamento seleccionado en el combo de departamentos
       
    }

    /**
     * Retorna el municipio seleccionado actualmente.
     * @return Municipio seleccionado.
     */
    public Municipio darMunicipioSeleccionado( )
    {
        Municipio m = ( Municipio )comboMunicipios.getSelectedItem( );
        return m;
    }

    /**
     * Retorna el puesto de votacin seleccionado actualmente.
     * @return Puesto de votacin seleccionado.
     */
    public PuestoVotacion darPuestoSeleccionado( )
    {
    	PuestoVotacion p = ( PuestoVotacion )comboPuestos.getSelectedItem( );
        return p;
    }

    /**
     * Actualiza la informacin de departamentos.
     */
    public void actualizarDepartamentos( )
    {
    	//TODO 5.D. Actualiza el combo de departamentos
    	//	Solicite los departamentos a la clase principal
    	//	Elimine cualquier item que exista en el combo
    	//	Agregue los nuevos items al combo

    }

    /**
     * Actualiza la informacin de municipios.
     * @param pDepartamento Departamento de los municipios. pDepartamento != null && pDepartamento != "".
     */
    public void actualizarMunicipios( Departamento pDepartamento )
    {
        
      List<Municipio> municipios = pDepartamento.darMunicipios();
      comboMunicipios.removeAllItems();
      for (Municipio municipio : municipios) {
		comboMunicipios.addItem(municipio);
      }

    }

    /**
     * Actualiza la informacin de los puestos de votacin.
     * @param pMunicipio Municipio de los puestos de votacin. pMunicipio != null && pMunicipio != "".
     */
    public void actualizarPuestos( Municipio pMunicipio )
    {
        List<PuestoVotacion> puestos = pMunicipio.darPuestosVotacion();
        comboPuestos.removeAllItems();
        for (PuestoVotacion puestoVotacion : puestos) {
			comboPuestos.addItem(puestoVotacion);
        	
		}
       
    }

    /**
     * Actualiza la lista de ciudadanos inscritos.
     * @param pPuesto Puesto de votacin de los ciudadanos. pPuesto != null && pPuesto != "".
     */
    public void actualizarListaInscritos( PuestoVotacion pPuesto )
    {
    	listaInscritos.removeAll();
    	listaInscritos.setListData(pPuesto.darInscritos().toArray());
        
        if( !pPuesto.darInscritos().isEmpty( ) )
        {
            listaInscritos.setSelectedIndex( 0 );
        }
    }

    /**
     * Actualiza la informacin del puesto de votacin seleccionado.
     * @param pPuestoNombre Nombre del puesto de votacin. pPuestoNombre != null && pPuestoNombre != "".
     * @param pPuestoDireccion Direccin del puesto de votacin. pPuestoDireccion != null && pPuestoDireccion != "".
     */
    public void actualizarInfoPuesto( String pPuestoNombre, String pPuestoDireccion )
    {
        if( pPuestoNombre != null )
        {
            txtNombre.setText( pPuestoNombre );
            txtDireccion.setText( pPuestoDireccion );
        }
        else
        {
            txtNombre.setText( "" );
            txtDireccion.setText( "" );
        }
    }

    /**
     * Manejo de los eventos de los botones.
     * @param pEvento Accin que gener el evento.
     */
    public void actionPerformed( ActionEvent pEvento )
    {
        String comando = pEvento.getActionCommand( );
        if( comando.equals( CAMBIO_DEPARTAMENTO ) )
        { 
          Departamento d = darDepartamentoSeleccionado();
          if(d != null)
          {
        	  actualizarMunicipios(d);
          }
        }
        else if( comando.equals( CAMBIO_MUNICIPIO ) )
        {
            Municipio m = darMunicipioSeleccionado();
            if( m != null )
            {
            	actualizarPuestos( m );
            	
            }
        }
        else if( comando.equals( CAMBIO_PUESTO ) )
        {
            PuestoVotacion p = darPuestoSeleccionado();
            if( p != null )
            {
                actualizarListaInscritos( p );
                actualizarInfoPuesto( p.darNombre( ), p.darDireccion( ) );
            }

        }
        else if( comando.equals( VER_INFO ) )
        {
            Ciudadano c = ( Ciudadano )listaInscritos.getSelectedValue( );

            if( c != null )
            {
                DialogoInformacionCiudadano dialogo = new DialogoInformacionCiudadano( c.darCedula( ), c.darApellidos( ), c.darNombres( ), c.darFechaNacimiento( ), c.darLugarNacimiento( ), c.darSexo( ), c.darFechaExpedicion( ),
                        c.darLugarExpedicion( ) );
                dialogo.setVisible( true );
            }
        }
    }
}
