/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.interfaz;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

/**
 * Panel de opciones del usuario.
 */
public class PanelOpciones extends JPanel implements ActionListener
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
     * Constante que representa el comando para inscribir a un ciudadano.
     */
    private static final String INSCRIBIR = "Inscribir";

    /**
     * Constante que representa el comando para generar estadsticas.
     */
    private static final String ESTADISTICAS = "Estadsticas";

    /**
     * Constante que representa el comando para importar informacin.
     */
    private static final String IMPORTAR = "Importar informacin";

    /**
     * Constante que representa el comando para generar reporte.
     */
    private static final String REPORTE = "Generar reporte";
    
    /**
     * Constante que representa el comando para generar reporte.
     */
    private static final String REPORTE_HTML = "Generar reporte HTML";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Ventana principal de la aplicacin.
     */
    private InterfazInscripcionCedulas principal;

    // -----------------------------------------------------------------
    // Atributos de interfaz
    // -----------------------------------------------------------------

    /**
     * Botn para inscribir a un ciudadano.
     */
    private JButton btnInscribir;

    /**
     * Botn para generar estadsticas.
     */
    private JButton btnEstadisticas;

    /**
     * Botn para importar informacin.
     */
    private JButton btnImportar;

    /**
     * Botn para generar reporte.
     */
    private JButton btnReporte;
    
    /**
     * Botn para generar reporte.
     */
    private JButton btnReporteHTML;
    
    /**
     * Estado de carga. True si est cargado, falso de lo contrario.
     */
    private boolean sistemaCargado;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Constructor del panel.<br>
     * @param pPrincipal Ventana principal. pPrincipal != null.
     */
    public PanelOpciones( InterfazInscripcionCedulas pPrincipal )
    {
        principal = pPrincipal;
        if( principal.darDepartamentos( ).size( ) == 0 )
        {
            sistemaCargado = false;
        }
        else
        {
            sistemaCargado = true;
        }
        setBorder( new TitledBorder( "" ) );
        setLayout( new GridLayout( 5, 1 ) );

        // Botn Inscribir Ciudadano
        btnInscribir = new JButton( );
        btnInscribir.setActionCommand( INSCRIBIR );
        btnInscribir.addActionListener( this );
        btnInscribir.setIcon( new ImageIcon( "./data/imagenes/inscribir.png" ) );
        btnInscribir.setToolTipText( "Inscribir ciudadano" );
       
        if(!sistemaCargado){
            btnInscribir.setEnabled( false );
        }
        add( btnInscribir );

        // Botn Generar Estadsticas
        btnEstadisticas = new JButton( );
        btnEstadisticas.setActionCommand( ESTADISTICAS );
        btnEstadisticas.addActionListener( this );
        btnEstadisticas.setIcon( new ImageIcon( "./data/imagenes/estadisticas.png" ) );
        btnEstadisticas.setToolTipText( "Grficos estadsticos" );
        if(!sistemaCargado){
            btnEstadisticas.setEnabled( false );
        }
        add( btnEstadisticas );

        // Botn Generar Reporte
        btnReporte = new JButton( );
        btnReporte.setActionCommand( REPORTE );
        btnReporte.addActionListener( this );
        btnReporte.setIcon( new ImageIcon( "./data/imagenes/reporte.png" ) );
        btnReporte.setToolTipText( "Generar reporte" );
        if(!sistemaCargado){
            btnReporte.setEnabled( false );
        }
        add( btnReporte );
        
        btnReporteHTML = new JButton( );
        btnReporteHTML.setActionCommand( REPORTE_HTML );
        btnReporteHTML.addActionListener( this );
        btnReporteHTML.setIcon( new ImageIcon( "./data/imagenes/reporteHTML.png" ) );
        btnReporteHTML.setToolTipText( "Generar reporte" );
        if(!sistemaCargado){
        	btnReporteHTML.setEnabled( false );
        }
        add( btnReporteHTML );

        // Botn Importar Datos
        btnImportar = new JButton( );
        btnImportar.setActionCommand( IMPORTAR );
        btnImportar.addActionListener( this );
        btnImportar.setIcon( new ImageIcon( "./data/imagenes/importar.png" ) );
        btnImportar.setToolTipText( "Importar datos" );
        add( btnImportar );
        if(sistemaCargado){
        	btnImportar.setEnabled( false );
        }
        asignarEstilo();
    }
    
    private void asignarEstilo()
    {
    	Color c = new Color(0, 152, 255);
    	Color d = new Color(150,150,150);
    	if(sistemaCargado)
    	{
    		btnEstadisticas.setBackground(c);
    		btnInscribir.setBackground(c);
    		btnImportar.setBackground(d);
    		btnReporte.setBackground(c);
    		btnReporteHTML.setBackground(c);
    	}
    	else
    	{
    		btnEstadisticas.setBackground(d);
    		btnInscribir.setBackground(d);
    		btnImportar.setBackground(c);
    		btnReporte.setBackground(d);
    		btnReporteHTML.setBackground(d);
    	}
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Manejo de los eventos de los botones. <br>
     * @param pEvento Accin que gener el evento.
     */
    public void actionPerformed( ActionEvent pEvento )
    {
        String comando = pEvento.getActionCommand( );
        if( comando.equals( INSCRIBIR ) )
        {
            principal.inscribirCiudadano( );
        }
        else if( comando.equals( ESTADISTICAS ) )
        {
            DialogoEstadisticas dialogo = new DialogoEstadisticas( principal );
            dialogo.setVisible( true );
        }
        else if( comando.equals( REPORTE ) )
        {
            principal.generarReporte( );
        }
        else if( comando.equals( REPORTE_HTML ) )
        {
            principal.generarReporteHTML( );
        }
        else if( comando.equals( IMPORTAR ) )
        {
            principal.importar( );
           if( principal.darDepartamentos( ).size( ) != 0 )
            {
                btnInscribir.setEnabled( true );
                btnEstadisticas.setEnabled( true );
                btnReporte.setEnabled( true );
                btnReporteHTML.setEnabled( true );
                btnImportar.setEnabled(false);
                asignarEstilo();
            }
            else
            {
                btnInscribir.setEnabled( false );
                btnEstadisticas.setEnabled( false );
                btnReporte.setEnabled( false );
                btnReporteHTML.setEnabled( false );
                btnImportar.setEnabled(true);
                asignarEstilo();
            }
           
        }
    }

    /**
     * Activa las opciones una vez cargados los puestos
     */
	public void activarOpciones() 
	{
		sistemaCargado = true;
		asignarEstilo();
		
	}
}
