package uniandes.cupi2.inscripcionCedulas.manejoArchivos;

import java.io.File;
import java.io.PrintWriter;

import uniandes.cupi2.inscripcionCedulas.excepciones.PersistenciaException;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;

/**
 * Clase para generar en formato HTMl
 * @author alvar-go
 *
 */
public class GeneradorReporteHTML 
{
	/**
	 * Genera un reporte en formato HTML del municipio que llega por parmetro
	 * post: Se ha generado el reporte en la carpeta indicada y con el nombre <nombremunicipio>.html
	 * @param municipio el municipio del que se desea generar el reporte. municipio != null
	 * @param carpeta la carpeta donde se generar el reporte. carpeta != null
	 * @throws PersistenciaException Se lanza en caso que no se posible escribir el archivo
	 */
	public static void generarReporte(Municipio municipio, String carpeta)throws PersistenciaException
	{
		try {
			File archivo = new File(carpeta);
			if(!archivo.exists())
			{
				archivo.mkdirs();
			}
			File reporte = new File(archivo, municipio.darNombre()+".html");
			PrintWriter out = new PrintWriter(reporte);
			out.println("<html>");
			out.println("<head>");
			imprimirEstilo(out);
			out.println("</head>");
			//TODO 16.A Complete escribiendo en el archivo el cuerpo del reporte HTML
			//		Revise cuidadosamente el ejemplo para identificar las clases que debe usar
			//		Recuerde que los puestos ya saben escribir su informacin en formato HTML

			out.println("</html>");
			out.close();
			
		} 
		catch (Exception e) {
			throw new PersistenciaException("No fue posible crear el reporte: " + e.getMessage());
		}
	}

	/**
	 * Agrega el estilo css para que el reporte luzca como se solicita en el enunciado
	 * @param out el canal por donde se escribe la informacin. out != null
	 */
	private static void imprimirEstilo(PrintWriter out) 
	{
		out.println("<style>");
		out.println("body{font-family: Helvetica Neue,Helvetica,Arial,sans-serif; }");
		out.println("h1.principal{color:#ffffff; margin-top: 5px; margin-left:5px;}");
		out.println("a{color: #0066ff; text-decoration: none;}");
		out.println("div.top{background: #0066ff; position:fixed; top: 0; left:0; height:60px; margin:0px auto; width:100%; z-index: 999;}");
		out.println("div.dpuesto{width:100%}");
		out.println("div.infopuesto{background:#f5f5f5;}");
		out.println("div.lateral{position: fixed;width: 30%;background-color: #ddd; font-size: 14px;overflow-y: scroll;top: 60px; bottom: 0; left:0;}");
		out.println("div.lateral>h3{color:#202121; margin-left:20px;}");
		out.println("div.lateral>ul{list-style-type:none;}");
		out.println("div.lateral>ul>li{margin-top: 10px;}");
		out.println("div.lateral>ul>li:hover{background: #f5f5f5;}");
		out.println("div.lateral>ul>li>a:hover{color: #663366;}");
		out.println("table.puesto{width:90%; margin-left:auto; margin-right: auto;}");
		out.println("table.puesto>thead{background: #0066ff; color:#ffffff;}");
		out.println("table.puesto>tbody{background: #ccc; font-size: 12px;}");
		out.println("div.central{position: relative; margin-left: 35%; padding-top:70px;}");
		out.println("</style>");
		
	}
}
