/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.mundo;

import java.io.PrintWriter;
import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import uniandes.cupi2.inscripcionCedulas.manejoArchivos.IReporteHTMLInscripcion;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.IReporteTextoInscripcion;

/**
 * Clase que representa un ciudadano inscrito en un puesto de votacin.<br>
 * <b>inv:</b><br>
 * TODO: 1.B. Documente la invariante
 */
public class Ciudadano implements IReporteTextoInscripcion, IReporteHTMLInscripcion
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------



    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Cdula del ciudadano.
     */
    private String cedula;

    /**
     * Apellidos del ciudadano.
     */
    private String apellidos;

    /**
     * Nombres del ciudadano.
     */
    private String nombres;

    /**
     * Fecha de nacimiento del ciudadano.
     */
    private Date fechaNacimiento;

    /**
     * Lugar de nacimiento del ciudadano.
     */
    private String lugarNacimiento;

    /**
     * Sexo del ciudadano.
     */
    private Genero sexo;

    /**
     * Fecha de expedicin de la cdula del ciudadano.
     */
    private Date fechaExpedicion;

    /**
     * Lugar de expedicin de la cdula del ciudadano.
     */
    private String lugarExpedicion;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Inicializa el ciudadano con la informacin dada por parmetro. <br>
     * <b>post: </b>El ciudadano se inicializ con el valor nombre dado por parmetro.<br>
     * @param pCedula Cdula del ciudadano. pCedula != null && pCedula != "".
     * @param pApellidos Apellidos del ciudadano. pApellidos != null && pApellidos != "".
     * @param pNombres Nombres del ciudadano. pNombres != null && pNombres != "".
     * @param pFechaNacimiento Fecha de nacimiento del ciudadano. pFechaNacimiento != null.
     * @param pLugarNacimiento Lugar de nacimiento del ciudadano. pLugarNacimiento != null && pLugarNacimiento != "".
     * @param pSexo Sexo del ciudadano. pSexo != null && (pSexo == Genero.FEMENINO || pSexo == Genero.MASCULINO ).
     * @param pFechaExpedicion Fecha de expedicin de la cdula del ciudadano. pFechaExpedicion != null.
     * @param pLugarExpedicion Lugar de expedicin de la cdula del ciudadano. pLugarExpedicion != null && pLugarExpedicion != "".
     */
    public Ciudadano( String pCedula, String pApellidos, String pNombres, Date pFechaNacimiento, String pLugarNacimiento, Genero pSexo, Date pFechaExpedicion, String pLugarExpedicion )
    {
        cedula = pCedula;
        apellidos = pApellidos;
        nombres = pNombres;
        fechaNacimiento = pFechaNacimiento;
        lugarNacimiento = pLugarNacimiento;
        sexo = pSexo;
        fechaExpedicion = pFechaExpedicion;
        lugarExpedicion = pLugarExpedicion;

    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Retorna la cdula del ciudadano.
     * @return Cdula del ciudadano.
     */
    public String darCedula( )
    {
        return cedula;
    }

    /**
     * Retorna los apellidos del ciudadano.
     * @return Apellidos del ciudadano.
     */
    public String darApellidos( )
    {
        return apellidos;
    }

    /**
     * Retorna los nombres del ciudadano.
     * @return Nombres del ciudadano.
     */
    public String darNombres( )
    {
        return nombres;
    }

    /**
     * Retorna la fecha de nacimiento del ciudadano.
     * @return Fecha de nacimiento del ciudadano.
     */
    public Date darFechaNacimiento( )
    {
        return fechaNacimiento;
    }

    /**
     * Retorna el lugar de nacimiento del ciudadano.
     * @return Lugar de nacimiento del ciudadano.
     */
    public String darLugarNacimiento( )
    {
        return lugarNacimiento;
    }

    /**
     * Retorna la edad en aos del ciudadano.
     * @return Edad del ciudadano.
     */
    public int darEdad( )
    {
        Date actual = new Date( );
        Calendar hoy = Calendar.getInstance();
        hoy.setTime(actual);
        
        Calendar nac = Calendar.getInstance();
        nac.setTime(fechaNacimiento);
        
        int edad = hoy.get(Calendar.YEAR) - nac.get(Calendar.YEAR);
        
        if( hoy.get(Calendar.MONTH) <= nac.get(Calendar.MONTH) && hoy.before(nac) )
        {
            edad -= 1;
        }
       

        return edad;
    }

    /**
     * Retorna el sexo del ciudadano.
     * @return Sexo del ciudadano.
     */
    public Genero darSexo( )
    {
        return sexo;
    }

    /**
     * Retorna la fecha de expedicin de la cdula del ciudadano.
     * @return Fecha de expedicin de la cdula del ciudadano.
     */
    public Date darFechaExpedicion( )
    {
        return fechaExpedicion;
    }

    /**
     * Retorna el lugar de expedicin de la cdula del ciudadano.
     * @return Lugar de expedicin de la cdula del ciudadano.
     */
    public String darLugarExpedicion( )
    {
        return lugarExpedicion;
    }
    
    /**
     * Indica si la edad del ciudadano estpa en el rango dado
     * @param edad el rango de edad que se desea verificar. edad != null && edad pertenece a Edad
     * @return true si la edad del cliente est en el rango dado o false en caso contrario
     */
    public boolean estaEnRangoEdad(RangoEdad edad)
    {
    	return darEdad() >= edad.darLimiteInferior() && darEdad() <= edad.darLimiteSuperior();
    }
    
    /**
     * Escribe la informacin del ciudadano en el canal de salida que llega por parmetro
     * Se sigue el formato presentado en el archivo de descripcin. 
     *  Cdula: <nmero de cdula ciudadano 1>
 		Fecha de expedicin: <fecha de expedicin ciudadano 1>
 		Lugar de expedicin: <lugar de expedicin ciudadano 1>
 		Nombre: <nombres ciudadano 1> <apellidos ciudadano 1>
 		Fecha de nacimiento: <fecha de nacimiento ciudadano 1>
 		Lugar de nacimiento: <lugar de nacimiento ciudadano 1>
 		Sexo: <Sexo ciudadano 1>
     *@param out canal de salida donde se debe escribir. out != null
     */
    public void escribirInformacionReporte(PrintWriter out) {
    	
    	//TODO 10.A Escriba la informacin del ciudadano en el canal out.
    	
    }
    
    /**
     * Escribe la informacin del ciuadano en el reporte HTML
     * Se debe escribir una fila de tabla HTML con la informacin del ciudadano
     * @param out el canal para escribir en el archivo
     */
    public void escribirInformacionReporteHTML(PrintWriter out) 
    {
    	//TODO 14.A Complete el mtodo de acuerdo a la documentacin
		
	}

    /**
     * Retorna la representacin en String del ciudadano.
     * @return Una cadena con los nombres, apellidos y cdula con el siguiente formato: <nombres> <apellidos> - <cdula>.
     */
    public String toString( )
    {
        return nombres + " " + apellidos + " - " + cedula;
    }


    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

	private void verificarInvariante()
	{
		//TODO 1.C Implemente el mtodo verificar invariante
	}
}