/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.inscripcionCedulas.mundo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import uniandes.cupi2.inscripcionCedulas.excepciones.CiudadanoRegistradoException;

/**
 * Clase que representa el sistema de inscripcin de cdulas.<br>
 * <b>inv:</b><br>
 * departamentos != null. <br>
 * ciudadanos != null. <br>
 * No existen dos o ms ciudadanos con el mismo nmero de cdula. <br>
 */
public class InscripcionCedulas
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------


    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Lista de departamentos.
     */
    private List<Departamento> departamentos;

    /**
     * Lista de ciudadanos inscritos en el sistema.
     */
    private List<Ciudadano> ciudadanos;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Construye el sistema de inscripcin de cdulas. <br>
     * <b>post: </b>Los departamentos y ciudadanos se crean vacos.<br>
     */
    public InscripcionCedulas( )
    {
        departamentos = new ArrayList<Departamento>( );
        ciudadanos = new ArrayList<Ciudadano>( );

    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Agrega un departamento a la lista de departamentos del sistema
     * pre:		La lista de departamentos se encuentra inicializada
     * @param departamento el departamento que se desea agregar a la lista. departamento != null
     */
    public void agregarDepartamento(Departamento departamento)
    {
    	departamentos.add(departamento);
    }
    /**
     * Retorna el nmero de ciudadanos inscritos en el sistema.
     * pre:		La lista de departamentos se encuentra inicializada
     * @return Nmero de ciudadanos inscritos en el sistema.
     */
    public int darNumInscritos( )
    {
        int numInscritos = 0;
        for (Departamento departamento : departamentos) {
			numInscritos += departamento.darNumInscritos();
		}

        return numInscritos;
    }

    /**
     * Retorna la lista de departamentos.
     * @return Departamentos del sistema de inscripcin de cdulas.
     */
    public List<Departamento> darDepartamentos( )
    {
        return departamentos;
    }

    

    /**
     * Inscribe a un ciudadano en el sistema.
     * post:	Se ha agregado el ciudadano a la lista de ciudadanos
     * @param pPuesto Puesto de votacin donde se inscribe el usuario. pPuesto != null y existe en el sistema.
     * @param pCedula Cdula del ciudadano. pCedula != null && pCedula != "".
     * @param pApellidos Apellidos del ciudadano. pApellidos != null && pApellidos != "".
     * @param pNombres Nombres del ciudadano. pNombres != null && pNombres != "".
     * @param pFechaNacimiento Fecha de nacimiento del ciudadano. pFechaNacimiento != null.
     * @param pLugarNacimiento Lugar de nacimiento del ciudadano. pLugarNacimiento != null && pLugarNacimiento != "".
     * @param pSexo Sexo del ciudadano. pSexo != null && (pSexo == FEMENINO || pSexo == MASCULINO ).
     * @param pFechaExpedicion Fecha de expedicin de la cdula del ciudadano. pFechaExpedicion != null.
     * @param pLugarExpedicion Lugar de expedicin de la cdula del ciudadano. pLugarExpedicion != null && pLugarExpedicion != "".
     * @throws CiudadanoRegistradoException Se lanza una excepcin si ya existe un ciudadano registrado con el nmero de cdula dado.
     */
    public void inscribirCiudadano( PuestoVotacion pPuesto, String pCedula, String pApellidos, String pNombres, Date pFechaNacimiento, String pLugarNacimiento, Genero pSexo, Date pFechaExpedicion,
            String pLugarExpedicion ) throws CiudadanoRegistradoException
    {
        
        //TODO 2.A Complete el mtodo segn la documentacin.
        //Tenga en cuenta que debe agregar al ciudadano en la lista de ciudadanos e inscribirlo en el puesto.

    }

    /**
     * Busca el departamento cuyo nombre es dado por parmetro.
     * @param pNombreDepartamento Nombre del departamento. pNombreDepartamento != null && pNombreDepartamento != "".
     * @return Departamento con el nombre dado.
     */
    public Departamento buscarDepartamento( String pNombreDepartamento )
    {
        Departamento departamentoBuscado = null;
        for( int i = 0; i < departamentos.size( ); i++ )
        {
            Departamento d = departamentos.get( i );
            if( d.darNombre( ).equals( pNombreDepartamento ) )
            {
                departamentoBuscado = d;
            }
        }

        return departamentoBuscado;
    }

    /**
     * Calcula el nmero de inscritos hasta el momento del gnero dado
     * pre:		La lista de departamentos se encuentra inicializada
     * @param g Gnero del que se desean conocer los inscritos. g != null
     * @return Nmero de inscritos del gnero dado.
     */
    public int calcularNumeroInscritosPorGenero( Genero g)
    {
        int numHombres = 0;
        for (Departamento departamento : departamentos) {
			numHombres += departamento.calcularNumeroInscritosPorGenero(g);
		}

        return numHombres;
    }


    /**
     * Calcula porcentaje de inscritos del gnero dado en el sistema.
     * pre:		La lista de ciudadanos se encuentra inicializada
     * 			La lisa de departamentos se encuentra inicializada
     * @return Porcentaje de inscritos del gnero dado en el sistema.
     */
    public double calcularPorcentajeInscritosGenero( Genero g)
    {
        int total = darNumInscritos( );
        if( total == 0 )
        {
            return 0.0;
        }

        int hombres = calcularNumeroInscritosPorGenero(g);
        return ( double )hombres / ( double )total;
    }


    /**
     * Calcula el nmero de ciudadanos inscritos en el rango de edad dado por parmetro.<br>
     * pre:		La lista de ciudadanos se encuentra inicializada
     * @param pRango Rango de edad. pRango != null 
     * @return Ciudadanos inscritos en el rango de edad dado.
     */
    public int calcularInscritosRangoEdad( RangoEdad pRango )
    {
        int respuesta = 0;
        for (Ciudadano ciudadano : ciudadanos) {
        	
			if(ciudadano.estaEnRangoEdad(pRango))
			{
				respuesta++;
			}
		}

        return respuesta;
    }
    /**
     * Verifica si est inscrito o no un ciudadano en el sistema.
     * pre:		La lista de inscritos se encuentra inicializada
     * @param pCedulaCiudadano Cdula del ciudadano que se desea saber si est inscrito o no en el puesto de votacin. pCedulaCiudadano != null && pCedulaCiudadano != "".
     * @return True si el ciudadano se encuentra inscrito en algn departamento o false de lo contrario.
     */
    public boolean existeCiudadano( String pCedulaCiudadano )
    {
        boolean seEncuentra = false;
        for( int i = 0; i < departamentos.size( ); i++ )
        {
            Departamento d = ( Departamento )departamentos.get( i );
            if( d.existeCiudadano( pCedulaCiudadano ) )
            {
                seEncuentra = true;
            }
        }
        return seEncuentra;
    }

    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

    /**
     * Verifica el invariante de la clase: <br>
     * <b>inv:</b><br>
     * departamentos != null. <br>
     * ciudadanos != null. <br>
     * No existen dos o ms ciudadanos con el mismo nmero de cdula. <br>
     */
    private void verificarInvariante( )
    {
        assert departamentos != null : "La lista de departamentos no puede ser nula.";
        assert ciudadanos != null : "La lista de ciudadanos  no puede ser nula.";
        assert existenCiudadanosConLaMismaCedula( ) == false : "No puden haber ciudadanos con el mismo nmero de cdula.";
    }

    /**
     * Revisa si existen dos ciudadanos con el mismo nmero de cdula.
     * @return True si existen dos ciudadanos con el mismo nmero de cdula, o false de los contrario.
     */
    private boolean existenCiudadanosConLaMismaCedula( )
    {
        boolean ciudadanoEncontrado = false;
        for( int i = 0; i < ciudadanos.size( ); i++ )
        {
            Ciudadano c1 = ( Ciudadano )ciudadanos.get( i );

            for( int j = i + 1; j < ciudadanos.size( ); j++ )
            {
                Ciudadano c2 = ( Ciudadano )ciudadanos.get( j );

                if( c1.equals( c2 ) )
                {
                    ciudadanoEncontrado = true;
                }
            }
        }

        return ciudadanoEncontrado;
    }

    // -----------------------------------------------------------------
    // Puntos de Extensin
    // -----------------------------------------------------------------

    /**
     * Mtodo para la extensin 1.
     * @return respuesta1.
     */
    public String metodo1( )
    {
        return "Respuesta 1";
    }

    /**
     * Mtodo para la extensin2.
     * @return respuesta2.
     */
    public String metodo2( )
    {
        return "Respuesta 2";
    }
}