/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.mundo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Clase que representa un municipio.<br>
 * <b>inv:</b><br>
 * nombre != null && nombre != "". <br>
 * puestosVotacion != null. <br>
 */
public class Municipio
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------


    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Nombre del municipio.
     */
    private String nombre;

    /**
     * Lista de puestos de votacin en el municipio.
     */
    private List<PuestoVotacion> puestosVotacion;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Inicializa el municipio con la informacin dada por parmetro.<br>
     * <b>post: </b>El municipio se inicializ con el valor nombre dado por parmetro.<br>
     * La lista de puestos de votacin qued inicializada.
     * @param pNombre Nombre del municipio. pNombre != null && pNombre != "".
     */
    public Municipio( String pNombre )
    {
        nombre = pNombre;
        puestosVotacion = new ArrayList<PuestoVotacion>( );

        verificarInvariante( );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Retorna el nombre del municipio.
     * @return Nombre del municipio.
     */
    public String darNombre( )
    {
        return nombre;
    }

    /**
     * Retorna la lista de puestos de votacin del municipio.
     * pre:		La lista de puestos se encuentra inicializada
     * @return Puesto de votacin del municipio.
     */
    public List<PuestoVotacion> darPuestosVotacion( )
    {
        return puestosVotacion;
    }

    /**
     * Retorna el nmero de ciudadanos inscritos en el municipio.
     * pre:		La lista de puestos se encuentra inicializada
     * @return Nmero de ciudadanos inscritos en el municipio.
     */
    public int darNumInscritos( )
    {
        int inscritos = 0;
        for (PuestoVotacion puestoVotacion : puestosVotacion) {
			inscritos += puestoVotacion.darNumInscritos();
		}

        return inscritos;
    }

    /**
     * Agrega un nuevo puesto de votacin del municipio.
     * pre:		La lista de puestos se encuentra inicializada
     * post:	Se ha agregado un puesto a la lista
     * @param pNombre Puesto de votacin a agregar. pNombre != null && pNombre != "".
     * @param pDireccion Puesto de votacin a agregar. pDireccion != null && pDireccion != "".
     */
    public void agregarPuestoVotacion( String pNombre, String pDireccion )
    {
        PuestoVotacion p = new PuestoVotacion( pNombre, pDireccion );
        puestosVotacion.add( p );
        verificarInvariante( );
    }

    /**
     * Inscribe a un ciudadano en el puesto de votacin del municipio dado por parmetro.
     * pre:		La lista de puestos se encuentra inidializada y el puesto existe en la lista
     * post:	Se ha agregado el ciudadano al puesto cuyo nombre llega por parmetro
     * @param pPuesto Nombre del puesto de votacin al que se va a inscribir el ciudadano. pPuesto != null && pPuesto != "".
     * @param pCiudadano Ciudadano a inscribir en el municipio. pCiudadano != null.
     */
    public void inscribirCiudadano( String pPuesto, Ciudadano pCiudadano )
    {
        PuestoVotacion p = buscarPuestoVotacion( pPuesto );
        p.inscribirCiudadano( pCiudadano );

        verificarInvariante( );
    }

    /**
     * Busca el puesto de votacin cuyo nombre es dado por parmetro.
     * pre:		La lista de puestos se encuentra inicializada
     * @param pNombrePuestoVotacion Nombre del puesto de votacin. pNombrePuestoVotacion != null && pNombrePuestoVotacion != "".
     * @return Puesto de votacin con el nombre dado, null en caso de no encontrarlo.
     */
    public PuestoVotacion buscarPuestoVotacion( String pNombrePuestoVotacion )
    {
        PuestoVotacion puestoBuscado = null;
        for( int i = 0; i < puestosVotacion.size( ); i++ )
        {
            PuestoVotacion p = puestosVotacion.get( i );
            if( p.darNombre( ).equals( pNombrePuestoVotacion ) )
            {
                puestoBuscado = p;
            }
        }
        return puestoBuscado;
    }

    /**
     * Calcula el nmero de inscritos del gnero que llega por parmetro en el municipio.
     * pre:		La lista de municipios se encuentra inicializada
     * @param g el genero del que se desea conocer la cantidad de inscritos. g != null
     * @return Nmero de inscritos del genero dado.
     */
    public int calcularNumeroInscritosPorGenero(Genero g )
    {
        int numHombre = 0;
        for( int i = 0; i < puestosVotacion.size( ); i++ )
        {
            PuestoVotacion p = ( PuestoVotacion )puestosVotacion.get( i );
            numHombre += p.calcularInscritosPorGenero(g );
        }

        return numHombre;
    }

    /**
     * Verifica si est inscrito o no un ciudadano en el municipio.<br>
     * pre:		La lista de puestos se encuentra inicializada
     * @param pCedulaCiudadano Cdula del ciudadano que se desea saber si est inscrito o no en el puesto de votacin. pCedulaCiudadano != null && pCedulaCiudadano != "".
     * @return True si el ciudadano se encuentra inscrito en algn puesto de votacin del municipio o false de lo contrario.
     */
    public boolean existeCiudadano( String pCedulaCiudadano )
    {

        boolean seEncuentra = false;
        for( int i = 0; i < puestosVotacion.size( ); i++ )
        {
            PuestoVotacion p =  puestosVotacion.get( i );
            if( p.existeCiudadano( pCedulaCiudadano ) )
            {
                seEncuentra = true;
            }
        }
        return seEncuentra;
    }

    /**
     * Retorna la representacin en String del municipio.
     * @return Una cadena con los nombres con el siguiente formato: <nombre>.
     */
    public String toString( )
    {
        return nombre;
    }

    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

    /**
     * Verifica el invariante de la clase: <br>
     * <b>inv:</b><br>
     * nombre != null && nombre != "". <br>
     * puestosVotacion != null. <br>
     */
    private void verificarInvariante( )
    {
        assert nombre != null && !nombre.equals( "" ) : "El nombre del municipio no puede ser nulo o vaco.";
        assert puestosVotacion != null : "La lista de puestos de votacin no puede ser nula.";
    }

}
