/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.mundo;

import java.io.PrintWriter;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import uniandes.cupi2.inscripcionCedulas.manejoArchivos.IReporteHTMLInscripcion;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.IReporteTextoInscripcion;

/**
 * Clase que representa un puesto de votacin.<br>
 * <b>inv:</b><br>
 *  El nombre no es nulo o cadena vaca
 *  La direccin no es nula o cadena vaca
 *  La lista de inscritos no es nula
 */
public class PuestoVotacion implements IReporteTextoInscripcion, IReporteHTMLInscripcion
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------


    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Nombre del puesto de votacin.
     */
    private String nombre;

    /**
     * Direccin del puesto de votacin.
     */
    private String direccion;

    /**
     * Lista de ciudadanos inscritos en el puesto de votacin.
     */
    private List<Ciudadano> inscritos;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Inicializa el puesto de votacin con la informacin dada por parmetro. <br>
     * <b>post: </b>El puesto de votacin se inicializ con los valores de nombre y direccin dados por parmetro.<br>
     * La lista de inscritos qued inicializada.
     * @param pNombre Nombre del puesto de votacin. pNombre != null && pNombre != "".
     * @param pDireccion Direccin del puesto de votacin. pDireccion != null && pDireccion != "".
     */
    public PuestoVotacion( String pNombre, String pDireccion )
    {
        nombre = pNombre;
        direccion = pDireccion;
        inscritos = new ArrayList<Ciudadano>( );
        verificarInvariante();

    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Retorna el nombre del puesto de votacin.
     * @return Nombre del puesto de votacin.
     */
    public String darNombre( )
    {
        return nombre;
    }

    /**
     * Retorna la lista de ciudadanos inscritos en el puesto de votacin.
     * pre:		La lista de inscritos se encuentra inicializada
     * @return Lista de ciudadanos inscritos.
     */
    public List<Ciudadano> darInscritos( )
    {
        return inscritos;
    }

    /**
     * Retorna la direccin del puesto de votacin.
     * @return Direccin del puesto de votacin.
     */
    public String darDireccion( )
    {
        return direccion;
    }

    /**
     * Retorna el nmero de ciudadanos inscritos en el puesto de votacin.
     * pre:		La lista de inscritos se encuentra inicializada
     * @return Nmero de ciudadanos inscritos en el puesto de votacin.
     */
    public int darNumInscritos( )
    {
        return inscritos.size( );
    }

    /**
     * Agrega un ciudadano a la lista de inscritos del puesto de votacin.
     * pre:		La lista de inscritos se encuentra inicializada
     * 			El ciudadano no existe en el puesto de votacin
     * post:	Se ha agregado el ciudadano a la lista de inscritos
     * @param pCiudadano Ciudadano a inscribir en el puesto de votacin. pCiudadano != null.
     */
    public void inscribirCiudadano( Ciudadano pCiudadano )
    {
        inscritos.add( pCiudadano );
        verificarInvariante();

    }
    
    /**
     * Calcula el nmero de inscritos hasta el momento del genero que llega por parmetro.
     * pre:		La lista de inscritos se encuentra inicializada
     * @param g el genero que se desea contar. g != null
     * @return Nmero de inscritos del genero dado.
     */
    public int calcularInscritosPorGenero(Genero g)
    {
    	 int cantidad = 0;
    	 for (Ciudadano ciudadano : inscritos) {
			if(ciudadano.darSexo().equals(g))
			{
				cantidad++;
			}
		}
         return cantidad;
    }

    /**
     * Verifica si est inscrito o no un ciudadano en el puesto de votacin.<br>
     * pre:		La lista de inscritos se encuentra inicializada
     * @param pCedulaCiudadano Cdula del ciudadano que se desea saber si est inscrito o no en el puesto de votacin. pCedulaCiudadano != null && pCedulaCiudadano != "".
     * @return Retorna true si el ciudadano se encuentra en la lista de inscrito o false de lo contrario.
     */
    public boolean existeCiudadano( String pCedulaCiudadano )
    {

        boolean seEncuentra = false;
        for( int i = 0; i < inscritos.size( ); i++ )
        {
            Ciudadano temp = ( Ciudadano )inscritos.get( i );
            if( temp.darCedula( ).equals( pCedulaCiudadano ) )
            {
                seEncuentra = true;
            }
        }
        return seEncuentra;
    }
    
    /**
     * Escribe la informacin del Departamento en el canal de salida que llega por parmetro
     * Se escribe siguiendo el formato del archivo de descripcin
     * 	Puesto de votacin: <nombre del puesto de votacin>
		Direccin: <direccin del puesto de votacin>
		=======================================================
		Informacin ciudadano 1
		=======================================================
		Informacin ciudadano 2
		=======================================================
		Informacin ciudadano k
		=======================================================
     * @param out canal de salida para escribir los datos. out != null
     */
    public void escribirInformacionReporte(PrintWriter out) {
    	//TODO 11.A Escriba la informacin del puesto en el canal out
    	//		Tenga en cuenta que cada ciudadano ya sabe escribir su informacin en un canal
    
    	
    }
    
    /**
     * Escribre la informacin del puesto en el reporte HTML
     * Debe crear un div con la informacin del puesto y la tabla de ciuadanos inscritos
     * @param out el canal para escribir en el archivo. out != null
     */
    public void escribirInformacionReporteHTML(PrintWriter out) 
	{
    	//TODO 15.A Complete el cuerto de acuerdo a la documentacin
    	//		Revise el ejemplo cuidadosamente para identificar las clases que css que debe asignar
    	//		Recuerde que los ciuadanos ya saben escribir su informacin en formato HTML

		
	}

    /**
     * Devuelve el id del puesto necesario para crear el reporte HTML
     * @return nombre sin espacios o puntos (.)
     */
	public String darId() 
	{
		return nombre.toLowerCase().replace(".", "").replace(" ", "");
	}
	
    /**
     * Devuelve la representacin en String del puesto de votacin. <br>
     * @return Una cadena con los nombres con el siguiente formato: <nombre>.
     */
    public String toString( )
    {
        return nombre;
    }


    // -----------------------------------------------------------------
    // Invariante
    // -----------------------------------------------------------------

	private void verificarInvariante()
	{
		assert(nombre != null && !nombre.equals(""));
		assert(direccion != null && !direccion.equals(""));
		assert(inscritos != null );
	}

	
}
