/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import junit.framework.TestCase;
import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;

/**
 * Clase que verifica la correcta implementacin de los mtodos de la clase Ciudadano.
 */
public class CiudadanoTest extends TestCase
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase donde se harn las pruebas.
     */
    private Ciudadano ciudadano;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Escenario 1: Construye un nuevo ciudadano.
     */
    public void setUp( )
    {
        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );
        calendar.set( anioActual - 22, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );

        calendar.set( anioActual - 3, mesActual, 15 );
        Date fechaExpedicion = calendar.getTime( );

        ciudadano = new Ciudadano( "123", "Prez", "Carlos", fechaNacimiento, "Bogot", Genero.MASCULINO, fechaExpedicion, "Bogot" );
    }

    /**
     * Prueba 1: Se encarga de verificar el mtodo constructor de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * darCedula<br>
     * darApellidos<br>
     * darNombres<br>
     * darFechaNacimiento<br>
     * darLugarNacimiento<br>
     * darSexo<br>
     * darEdad<br>
     * darFechaExpedicion<br>
     * darLugarExpedicion<br>
     * Ciudadano<br>
     * <b> Caso de prueba 1:</b><br>
     * Construye correctamente el ciudadano, cada uno de los valores corresponde al esperado.<br>
     */
    public void testCiudadano( )
    {

        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );
        calendar.set( anioActual - 22, mesActual, diaActual );

        

        assertEquals( "La cdula del ciudadano no es la esperada.", "123", ciudadano.darCedula( ) );
        assertEquals( "Los apellidos del ciudadanos no son los esperados.", "Prez", ciudadano.darApellidos( ) );
        assertEquals( "Los nombres del ciudadano no son los esperados.", "Carlos", ciudadano.darNombres( ) );
        
        Calendar c = Calendar.getInstance();
        c.setTime(ciudadano.darFechaNacimiento());
        assertEquals( "El ao de nacimiento del ciudadano no es el esperado.", calendar.get(Calendar.YEAR), c.get(Calendar.YEAR) );
        assertEquals( "El mes de nacimiento del ciudadano no es el esperado.", calendar.get(Calendar.MONTH), c.get(Calendar.MONTH)  );
        assertEquals( "El da de nacimiento del ciudadano no es el esperado.", calendar.get(Calendar.DAY_OF_MONTH), c.get(Calendar.DAY_OF_MONTH)  );
        
        assertEquals( "El lugar de nacimiento del ciudadano no es el esperado.", "Bogot", ciudadano.darLugarNacimiento( ) );
        assertEquals( "El sexo del ciudadano no es el esperado.", Genero.MASCULINO, ciudadano.darSexo( ) );
        
        calendar.set( anioActual - 3, mesActual, 15 );
        c.setTime(ciudadano.darFechaExpedicion());
        assertEquals( "El ao de expedicin del ciudadano no es el esperado.", calendar.get(Calendar.YEAR), c.get(Calendar.YEAR) );
        assertEquals( "El mes de expedicin del ciudadano no es el esperado.", calendar.get(Calendar.MONTH), c.get(Calendar.MONTH)  );
        assertEquals( "El da de expedicin del ciudadano no es el esperado.", calendar.get(Calendar.DAY_OF_MONTH), c.get(Calendar.DAY_OF_MONTH)  );
        
        assertEquals( "El lugar de expedicin de la cdula del ciudadano no es el esperado.", "Bogot", ciudadano.darLugarExpedicion( ) );
        assertEquals( "La edad del ciudadano no es la esperada.", 22, ciudadano.darEdad( ) );
    }
    
    /**
     * Prueba 2: Se encarga de verificar el mtodo escribirInformacionReporte de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * EscribirInformacionReporte<br>
     * <b> Casos de prueba:</b><br>
     * 1. El archivo generado es correcto.<br>
     */
    public void testEscribirInformacionReporte( )
    {
    	File f = new File("test/data/ciudadano.txt");
    	if(!f.exists())
    	{
    		try {
    			f.createNewFile();
				
			} catch (Exception e) {
				fail("No debera ocurrir problema creando el archivo: " + e.getMessage());
			}
    	}
    	try {
    		PrintWriter out = new PrintWriter(f);
    		ciudadano.escribirInformacionReporte(out);
    		out.close();
    		
    		BufferedReader in = new BufferedReader(new FileReader(f));
    		String linea = in.readLine();
    		
    		DateFormat df = new SimpleDateFormat("dd-MM-yyyy");
    		
    		assertNotNull("Debera existir la lnea de la cedula",linea);
    		assertTrue("La lnea cdula no tiene el formato esperado", linea.contains(ciudadano.darCedula()));
    		
    		linea = in.readLine();
    		assertNotNull("Debera existir la lnea de la fecha de expedicin",linea);
    		assertTrue("La lnea cdula no tiene el formato esperado", linea.contains(df.format(ciudadano.darFechaExpedicion())));
    		
    		linea = in.readLine();
    		assertNotNull("Debera existir la lnea de lugar de expedicin",linea);
    		assertTrue("La lnea cdula no tiene el formato esperado", linea.contains(ciudadano.darLugarExpedicion()));
    		
    		linea = in.readLine();
    		assertNotNull("Debera existir la lnea de nombre",linea);
    		assertTrue("La lnea cdula no tiene el formato esperado", linea.contains(ciudadano.darNombres()) && linea.contains(ciudadano.darApellidos()));
    		
    		linea = in.readLine();
    		assertNotNull("Debera existir la lnea de la fecha de nacimiento",linea);
    		assertTrue("La lnea cdula no tiene el formato esperado", linea.contains(df.format(ciudadano.darFechaNacimiento())));
    		
    		linea = in.readLine();
    		assertNotNull("Debera existir la lnea del sexo",linea);
    		assertTrue("La lnea cdula no tiene el formato esperado", linea.contains(ciudadano.darSexo().darNombre()));
			in.close();
			
			f.delete();
    		
    		
		} catch (Exception e) {
			fail("No dbera ocurrir problema escribiendo el archivo: " + e.getMessage());
		}
    }
    
    /**
     * Prueba 3: Se encarga de verificar el mtodo toString de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * toString<br>
     * <b> Casos de prueba:</b><br>
     * 1. El formato de texto es el correcto.<br>
     */
    public void testToString( )
    {
       
        assertEquals( "El formato del toString no es el esperado.", ciudadano.darNombres( ) + " " + ciudadano.darApellidos( ) + " - " + ciudadano.darCedula( ), ciudadano.toString( ) );
    }

}