/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.test;

import java.util.Calendar;
import java.util.Date;

import junit.framework.TestCase;
import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Departamento;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;

/**
 * Clase que verifica la correcta implementacin de la clase Departamento.
 */
public class DepartamentoTest extends TestCase
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase donde se harn las pruebas.
     */
    private Departamento departamento1;
    
    /**
     * Clase donde se harn las pruebas.
     */
    private Departamento departamento2;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Escenario 1: Construye un nuevo departamento vaco.
     * * Escenario 2: Construye un nuevo departamento con municipios, puestos de votacin y ciudadanos inscritos.
     */
    public void setUp( )
    {
        departamento1 = new Departamento( "departamento" );
        
        departamento2 = new Departamento( "departamento" );

        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        calendar.set( anioActual - 30, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c2 = new Ciudadano( "456", "apellidos2", "nombre2", fechaNacimiento, "nacimiento2", Genero.FEMENINO, fechaExpedicion, "expedicion2" );

        calendar.set( anioActual - 20, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c3 = new Ciudadano( "789", "apellidos3", "nombre3", fechaNacimiento, "nacimiento3", Genero.MASCULINO, fechaExpedicion, "expedicion3" );

        departamento2.agregarMunicipio( "municipio1" );
        departamento2.agregarMunicipio( "municipio2" );
        departamento2.agregarMunicipio( "municipio3" );

        Municipio m1 = departamento2.buscarMunicipio( "municipio1" );
        Municipio m2 = departamento2.buscarMunicipio( "municipio2" );
        Municipio m3 = departamento2.buscarMunicipio( "municipio3" );

        m1.agregarPuestoVotacion( "puesto1", "dir1" );
        m2.agregarPuestoVotacion( "puesto2", "dir2" );
        m3.agregarPuestoVotacion( "puesto3", "dir3" );

        m1.buscarPuestoVotacion( "puesto1" ).inscribirCiudadano( c1 );
        m2.buscarPuestoVotacion( "puesto2" ).inscribirCiudadano( c2 );
        m3.buscarPuestoVotacion( "puesto3" ).inscribirCiudadano( c3 );
    }


    /**
     * Prueba 1: Se encarga de verificar el mtodo constructor de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * darMunicipios<br>
     * darNombre<br>
     * darNumInscritos<br>
     * Departamento<br>
     * <b> Caso de prueba 1:</b><br>
     * Construye correctamente el departamento, cada uno de los valores corresponde al esperado.<br>
     */
    public void testDepartamento( )
    {

        assertNotNull( "La lista de municipios no ha sido inicializada.", departamento1.darMunicipios( ) );
        assertEquals( "El nombre del departamento no es el esperado.", "departamento", departamento1.darNombre( ) );
        assertEquals( "El nmero de ciudadanos inscritos no es el esperado.", 0, departamento1.darNumInscritos( ) );
    }

    /**
     * Prueba 2: Se encarga de verificar el mtodo agregarMunicipio de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * agregarMunicipio<br>
     * darMunicipios<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay municipios en el departamento.<br>
     */
    public void testAgregarMunicipio( )
    {
        Municipio m1 = new Municipio( "municipio1" );
        departamento1.agregarMunicipio( m1.darNombre( ) );

        assertFalse( "No se ha agregado el municipio al departamento.", departamento1.darMunicipios( ).isEmpty( ) );
        Municipio m = departamento1.darMunicipios( ).get( 0 );
        assertNotNull( "El municipio no fue agregado al departamento.", m );
    }

    /**
     * Prueba 3: Se encarga de verificar el mtodo buscarMunicipio de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarMunicipio<br>
     * <b> Casos de prueba:</b><br>
     * 1. El departamento no est vaco y tiene municipios.<br>
     * 2. El departamento est vaco y no posee municipios.<br>
     */
    public void testBuscarMunicipioOK( )
    {

        // 1
        Municipio municipio = departamento2.buscarMunicipio( "municipio2" );
        assertNotNull( "Se debera encontrar el municipio.", municipio );

        // 2
        Municipio municipio2 = departamento2.buscarMunicipio( "municipio" );
        assertNull( "No se debera encontrar el municipio.", municipio2 );

    }

    /**
     * Prueba 4: Se encarga de verificar el mtodo inscribirCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * inscribirCiudadano<br>
     * agregarMunicipio<br>
     * agregarPuestoVotacion<br>
     * existeCiudadano<br>
     * darCedula<br>
     * darApellidos<br>
     * darNombres<br>
     * darFechaNacimiento<br>
     * darLugarNacimiento<br>
     * darSexo<br>
     * darEdad<br>
     * darFechaExpedicion<br>
     * darLugarExpedicion<br>
     * darNumInscritos<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay municipios ni ciudadanos en el departamento.<br>
     */
    public void testInscribirCiudadano( )
    {
        
        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        assertEquals("No debera haber municipios", 0, departamento1.darMunicipios().size());
        departamento1.agregarMunicipio( "municipio1" );
        assertEquals("Debera existir un municipio", 1, departamento1.darMunicipios().size());

        Municipio m1 = departamento1.buscarMunicipio( "municipio1" );
        m1.agregarPuestoVotacion( "puesto1", "dir1" );

      //TODO 3.A Complete el agregando el ciudadano al puesto1 y probando que se agreg correctamento
        
    }

    /**
     * Prueba 5: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El departamento est vaco por ende, no tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoError1( )
    {

        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = departamento1.existeCiudadano( c1.darCedula( ) );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 6: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El departamento no est vaco y no tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoError2( )
    {

        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "234", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = departamento2.existeCiudadano( c1.darCedula( ) );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 7: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * <b> Casos de prueba:</b><br>
     * 1. El departamento no est vaco y tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoOK( )
    {
        boolean existe = departamento2.existeCiudadano( "123" );
        assertTrue( "Se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 8: Se encarga de verificar el mtodo calcularNumeroInscritosPorGenero de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * calcularNumeroInscritosPorGenero<br>
     * <b> Casos de prueba:</b><br>
     * 1. El departamento tiene ciudadanos hombres inscritos.<br>
     * 2. El departamento tiene ciudadanos muejres inscritas.<br>
     */
    public void calcularNumeroInscritosPorGenero( )
    {

        int hombres = departamento2.calcularNumeroInscritosPorGenero(Genero.MASCULINO);
        assertEquals( "El nmero de hombres no es el esperado.", 2, hombres );
        int mujeres = departamento2.calcularNumeroInscritosPorGenero(Genero.FEMENINO);
        assertEquals( "El nmero de mujeres no es el esperado.", 1, mujeres );
    }


    /**
     * Prueba 9: Se encarga de verificar el mtodo toString de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * toString<br>
     * <b> Casos de prueba:</b><br>
     * 1. El formato de texto es el correcto.<br>
     */
    public void testToString( )
    {
        assertEquals( "El formato del toString no es el esperado.", departamento1.darNombre( ), departamento1.toString( ) );
    }
}