package uniandes.cupi2.inscripcionCedulas.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import uniandes.cupi2.inscripcionCedulas.excepciones.PersistenciaException;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.GeneradorReportePuestoTXT;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.IReporteTextoInscripcion;
import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Departamento;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;
import junit.framework.TestCase;

/**
 * Clase que se encarga de probar el generador de reportes en archivos de texto
 * @author alvar-go
 *
 */
public class GeneradorReportePuestoTXTTest extends TestCase
{
	/**
	 * Ruta donde se generar el reporte
	 */
	public final static String RUTA = "test/data/reportetxttest.txt";
	
	/**
	 * Puesto de votacin del que se gener el reporte
	 */
	private PuestoVotacion puesto;
	
	/**
	 * Municipio del que se gener el reporte
	 */
	private Municipio municipio;
	
	/**
	 * Departamento del que se generar el reporte
	 */
	private Departamento departamento;
	
	/**
	 * Escenario 1. Crea un departamento, un municipio, un puesto y 3 ciudadanos
	 */
	public void setUp()
	{
		departamento = new Departamento("departamento1");
		municipio = new Municipio("municipio1");
		puesto = new PuestoVotacion( "puesto", "dir" );

        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        calendar.set( anioActual - 30, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c2 = new Ciudadano( "456", "apellidos2", "nombre2", fechaNacimiento, "nacimiento2", Genero.FEMENINO, fechaExpedicion, "expedicion2" );

        calendar.set( anioActual - 20, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c3 = new Ciudadano( "789", "apellidos3", "nombre3", fechaNacimiento, "nacimiento3", Genero.MASCULINO, fechaExpedicion, "expedicion3" );

        puesto.inscribirCiudadano( c1 );
        puesto.inscribirCiudadano( c2 );
        puesto.inscribirCiudadano( c3 );
	}
	
    /**
     * Prueba 1: Se encarga de verificar el mtodo generarReportePuesto de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * generarReportePuesto<br>
     * <b> Caso de prueba 1:</b><br>
     * La ubicacin del archivo es vlido y se puede generar el reporte.<br>
     */
	public void testGenerarReportePuesto()
	{
		try {
			GeneradorReportePuestoTXT.generarReportePuesto(departamento, municipio, puesto, RUTA);
			
			File f = new File(RUTA);
			assertTrue("Se debera haber creado el archivo",f.exists());
			try {
				BufferedReader in = new BufferedReader(new FileReader(f));
				
				String linea = in.readLine();
				assertNotNull("La lnea 1 debera existir", linea);
				assertTrue("La lnea 1 es incorrecta", linea.contains(departamento.darNombre()));
				
				linea = in.readLine();
				assertNotNull("La lnea 2 debera existir", linea);
				assertTrue("La lnea 2 es incorrecta", linea.contains(municipio.darNombre()));
				
				
				linea = in.readLine();
				assertNotNull("La lnea 3 debera existir", linea);
				assertTrue("La lnea 3 es incorrecta", linea.contains(puesto.darNombre()));
				
				linea = in.readLine();
				assertNotNull("La lnea 4 debera existir", linea);
				assertTrue("La lnea 4 es incorrecta", linea.contains(puesto.darDireccion()));
				
				int cont = 4;
				DateFormat df = new SimpleDateFormat("dd-MM-yyyy");
				
				for (Ciudadano c : puesto.darInscritos()) 
				{
					linea = in.readLine();
					cont++;
					assertNotNull("La lnea " + cont + " debera existir", linea);
					assertEquals("La lnea " + cont + " es incorrecta", IReporteTextoInscripcion.LINEA_PUNTEADA, linea);
					
					linea = in.readLine();
					cont++;
					assertNotNull("La lnea " + cont + " debera existir", linea);
					assertTrue("La lnea " + cont + " es incorrecta", linea.contains(c.darCedula()));
					
					linea = in.readLine();
					cont++;
					assertNotNull("La lnea " + cont + " debera existir", linea);
					assertTrue("La lnea " + cont + " es incorrecta", linea.contains(df.format(c.darFechaExpedicion())));
					
					linea = in.readLine();
					cont++;
					assertNotNull("La lnea " + cont + " debera existir", linea);
					assertTrue("La lnea " + cont + " es incorrecta", linea.contains(c.darLugarExpedicion()));
					
					
					linea = in.readLine();
					cont++;
					assertNotNull("La lnea " + cont + " debera existir", linea);
					assertTrue("La lnea " + cont + " es incorrecta", linea.contains(c.darNombres()) && linea.contains(c.darApellidos()));
					
					linea = in.readLine();
					cont++;
					assertNotNull("La lnea " + cont + " debera existir", linea);
					assertTrue("La lnea " + cont + " es incorrecta", linea.contains(df.format(c.darFechaNacimiento())));
					
					linea = in.readLine();
					cont++;
					assertNotNull("La lnea " + cont + " debera existir", linea);
					assertTrue("La lnea " + cont + " es incorrecta", linea.contains(c.darSexo().darNombre()));
				}
				linea = in.readLine();
				cont++;
				assertNotNull("La lnea " + cont + " debera existir", linea);
				assertEquals("La lnea " + cont + " es incorrecta", IReporteTextoInscripcion.LINEA_PUNTEADA, linea);

				in.close();
				
				f.delete();
			} catch (Exception e) {
				fail("No debera haber problema al tratar de leer el archivo: " + e.getMessage());
			}
			
		} catch (PersistenciaException e) {
			fail("No deberan ocurrir errores al generar el reporte: " + e.getMessage());
		}
	}
}
