/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.inscripcionCedulas.test;

import java.util.Calendar;
import java.util.Date;

import junit.framework.TestCase;
import uniandes.cupi2.inscripcionCedulas.excepciones.CiudadanoRegistradoException;
import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Departamento;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;
import uniandes.cupi2.inscripcionCedulas.mundo.InscripcionCedulas;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;
import uniandes.cupi2.inscripcionCedulas.mundo.RangoEdad;

/**
 * Clase usada para verificar que los mtodos de la clase InscripcionCedulas estn correctamente implementados.
 */
public class InscripcionCedulasTest extends TestCase
{

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase donde se harn las pruebas.
     */
    private InscripcionCedulas inscripcionCedulas1;
    
    /**
     * Clase donde se harn las pruebas.
     */
    private InscripcionCedulas inscripcionCedulas2;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Escenario 1: Construye un nuevo sistema InscripcionCedulas vaco.  
     * Escenario 2: Construye un nuevo sistema InscripcionCedulas con departamentos, municipios, puesto de votacin y ciudadanos.
     */
    public void setUp( )
    {
        inscripcionCedulas1 = new InscripcionCedulas();
        
        inscripcionCedulas2 = new InscripcionCedulas();
       
        Departamento d1 = new Departamento("departamento1");
        Departamento d2 = new Departamento("departamento2");
        
        d1.agregarMunicipio( "municipio1" );
        d2.agregarMunicipio( "municipio2" );
        d2.agregarMunicipio( "municipio3" );

        Municipio m1 = d1.buscarMunicipio( "municipio1" );
        Municipio m2 = d2.buscarMunicipio( "municipio2" );
        Municipio m3 = d2.buscarMunicipio( "municipio3" );

        m1.agregarPuestoVotacion( "puesto1", "dir1" );
        m2.agregarPuestoVotacion( "puesto2", "dir2" );
        m3.agregarPuestoVotacion( "puesto3", "dir3" );
        
        PuestoVotacion p1 = m1.buscarPuestoVotacion("puesto1");
        PuestoVotacion p2 = m2.buscarPuestoVotacion("puesto2");
        PuestoVotacion p3 = m3.buscarPuestoVotacion("puesto3");
        
        inscripcionCedulas2.agregarDepartamento(d1);
        inscripcionCedulas2.agregarDepartamento(d2);
        
        try {
	    	 Calendar calendar = Calendar.getInstance( );
	         int anioActual = calendar.get( Calendar.YEAR );
	         int mesActual = calendar.get( Calendar.MONTH );
	         int diaActual = calendar.get( Calendar.DAY_OF_MONTH );
	
	         calendar.set( anioActual - 50, mesActual, diaActual );
	         Date fechaNacimiento = calendar.getTime( );
	         calendar.set( anioActual, 2, 1 );
	         Date fechaExpedicion = calendar.getTime( );
        	inscripcionCedulas2.inscribirCiudadano(p1, "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1");
        	
        	calendar.set( anioActual - 30, mesActual, diaActual );
            fechaNacimiento = calendar.getTime( );
        	inscripcionCedulas2.inscribirCiudadano(p2, "456", "apellidos2", "nombre2", fechaNacimiento, "nacimiento2", Genero.FEMENINO, fechaExpedicion, "expedicion2");
        	
        	calendar.set( anioActual - 20, mesActual, diaActual );
            fechaNacimiento = calendar.getTime( );
        	inscripcionCedulas2.inscribirCiudadano(p3, "789", "apellidos3", "nombre3", fechaNacimiento, "nacimiento3", Genero.MASCULINO, fechaExpedicion, "expedicion3");
			
		} catch (Exception e) {
			e.printStackTrace();
			fail("No debera haber problema agreando ciudadanos");
		}
    }

    /**
     * Prueba 1: Se encarga de verificar el mtodo constructor de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * darDepartamentos<br>
     * darNumInscritos<br>
     * PuestoVotacion<br>
     * <b> Caso de prueba 1:</b><br>
     * Construye correctamente el sistema de inscripcin de cdulas, cada uno de los valores corresponde al esperado.<br>
     */
    public void testInscripcionCedulas( )
    {
        assertNotNull( "La lista de departamento no ha sido inicializada.", inscripcionCedulas1.darDepartamentos( ) );
        assertEquals( "El nmero de ciudadanos inscritos no es el esperado.", 0, inscripcionCedulas1.darNumInscritos( ) );
    }

    /**
     * Prueba 2: Se encarga de verificar el mtodo buscarDepartamento de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarDepartamento<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema no est vaco y tiene departamentos.<br>
     */
    public void testBuscarDepartamentoOK( )
    {
        Departamento departamento = inscripcionCedulas2.buscarDepartamento( "departamento1" );
        assertNotNull( "Se debera encontrar el departamento.", departamento );
    }

    /**
     * Prueba 3: Se encarga de verificar el mtodo buscarDepartamento de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarDepartamento<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema est vaca por ende, no tiene departamentos.<br>
     */
    public void testBuscarDepartamentoError1( )
    {
        Departamento departamento = inscripcionCedulas1.buscarDepartamento( "departamento1" );
        assertNull( "No se debera encontrar el departamento.", departamento );
    }

    /**
     * Prueba 4: Se encarga de verificar el mtodo buscarDepartamento de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarDepartamento<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema no est vaca y no tiene departamentos.<br>
     */
    public void testBuscarDepartamentoError2( )
    {
        Departamento departamento = inscripcionCedulas2.buscarDepartamento( "departamento" );
        assertNull( "No se debera encontrar el departamento.", departamento );
    }

    /**
     * Prueba 5: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema no est vaca y el ciudadano se encuentra inscrito.<br>
     */
    public void testExisteCiudadanoOK( )
    {
        boolean existe = inscripcionCedulas2.existeCiudadano( "123" );
        assertTrue( "Se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 6: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema est vaca por ende, no tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoError1( )
    {
        boolean existe = inscripcionCedulas1.existeCiudadano( "123" );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 7: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema no est vaca y el ciudadano no se encuentra inscrito.<br>
     */
    public void testExisteCiudadanoError2( )
    {
        boolean existe = inscripcionCedulas2.existeCiudadano( "987" );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 8: Se encarga de verificar el mtodo inscribirCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * inscribirCiudadano<br>
     * darCedula<br>
     * darApellidos<br>
     * darNombres<br>
     * darInscritos<br>
     * darFechaNacimiento<br>
     * darLugarNacimiento<br>
     * darSexo<br>
     * darEdad<br>
     * darFechaExpedicion<br>
     * darLugarExpedicion<br>
     * existeCiudadano<br>
     * darNumInscritos<br>
     * buscarDepartamento<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay ciudadanos en el sistema.<br>
     */
    public void testInscribirCiudadanoOK( )
    {

        Calendar calendar = Calendar.getInstance( );
        calendar.set( 1950, 5, 15 );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( 1980, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "987", "apellidos4", "nombre4", fechaNacimiento, "nacimiento4", Genero.MASCULINO, fechaExpedicion, "expedicion4" );

        try
        {
        	Departamento d = inscripcionCedulas2.buscarDepartamento("departamento1");
        	Municipio m = d.buscarMunicipio("municipio1");
        	PuestoVotacion p = m.buscarPuestoVotacion("puesto1");
        	
        }
        catch( CiudadanoRegistradoException e )
        {
        	//TODO 4.A Complete la prueba  verificando que no hay problema
          
        }

    }

    /**
     * Prueba 9: Se encarga de verificar el mtodo inscribirCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * inscribirCiudadano<br>
     * darCedula<br>
     * darApellidos<br>
     * darNombres<br>
     * darInscritos<br>
     * darFechaNacimiento<br>
     * darLugarNacimiento<br>
     * darSexo<br>
     * darEdad<br>
     * darFechaExpedicion<br>
     * darLugarExpedicion<br>
     * <b> Caso de prueba 1:</b><br>
     * Ya existe el ciudadano en el sistema.<br>
     */
    public void testInscribirCiudadanoError( )
    {
        Calendar calendar = Calendar.getInstance( );
         int anioActual = calendar.get( Calendar.YEAR );
         int mesActual = calendar.get( Calendar.MONTH );
         int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

         calendar.set( anioActual - 50, mesActual, diaActual );
         Date fechaNacimiento = calendar.getTime( );
         calendar.set( anioActual, 2, 1 );
         Date fechaExpedicion = calendar.getTime( );
         Departamento d = inscripcionCedulas2.buscarDepartamento("departamento1");
     	Municipio m = d.buscarMunicipio("municipio1");
     	PuestoVotacion p = m.buscarPuestoVotacion("puesto1");
    	try 
    	{
			//TODO 4.B Complete la prueba inscribiendo el ciudadano y verificando que se genera un problema
    	} 
    	catch (CiudadanoRegistradoException e) 
    	{
    		//TODO 4.B Complete la prueba verificando que no se inscribi al ciudadano
    		
    	}
    }

    /**
     * Prueba 10: Se encarga de verificar el mtodo calcularNumeroInscritosPorGenero de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * calcularNumeroInscritosPorGenero<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema tiene ciudadanos hombres inscritos.<br>
     * 2. El sistema tiene ciudadanos hombres inscritos.<br>
     */
    public void testCalcularNumeroInscritosPorGenero( )
    {

        int hombres = inscripcionCedulas2.calcularNumeroInscritosPorGenero(Genero.MASCULINO);
        assertEquals( "El nmero de hombres no es el esperado.", 2, hombres );
        
        int mujeres = inscripcionCedulas2.calcularNumeroInscritosPorGenero(Genero.FEMENINO);
        assertEquals( "El nmero de mujeres no es el esperado.", 1, mujeres );
    }

    /**
     * Prueba 11: Se encarga de verificar el mtodo calcularPorcentajeInscritosGenero de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * calcularPorcentajeInscritosGenero<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema tiene ciudadanos hombres inscritos.<br>
     * 2. El sistema tiene ciudadanas mujeres inscritas.<br>
     */
    public void testCalcularPorcentajeInscritosGenero( )
    {

        double hombres = inscripcionCedulas2.calcularPorcentajeInscritosGenero(Genero.MASCULINO);
        assertEquals( "El porcentaje de hombres no es el esperado.", ( double )2 / 3, hombres, 0.1 );
        
        double mujeres = inscripcionCedulas2.calcularPorcentajeInscritosGenero( Genero.FEMENINO );
        assertEquals( "El nmero de mujeres no es el esperado.", ( double )1 / 3, mujeres, 0.1 );

    }


    /**
     * Prueba 14: Se encarga de verificar el mtodo calcularInscritosRangoEdad de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * calcularInscritosRangoEdad<br>
     * <b> Casos de prueba:</b><br>
     * 1. El sistema tiene ciudadanos de diferentes edades.<br>
     */
    public void testCalcularInscritosRangoEdad( )
    {
        assertEquals( "El nmero de inscritos en el rango 1 (18 - 25 aos) no es el esperado.", 1, inscripcionCedulas2.calcularInscritosRangoEdad( RangoEdad.RANGO_0 ) );
        assertEquals( "El nmero de inscritos en el rango 2 (26 - 29 aos) no es el esperado.", 0, inscripcionCedulas2.calcularInscritosRangoEdad( RangoEdad.RANGO_1 ) );
        assertEquals( "El nmero de inscritos en el rango 3 (30 - 37 aos) no es el esperado.", 1, inscripcionCedulas2.calcularInscritosRangoEdad( RangoEdad.RANGO_2 ) );
        assertEquals( "El nmero de inscritos en el rango 4 (38 - 49 aos) no es el esperado.", 0, inscripcionCedulas2.calcularInscritosRangoEdad( RangoEdad.RANGO_3 ) );
        assertEquals( "El nmero de inscritos en el rango 5 (50 - 66 aos) no es el esperado.", 1, inscripcionCedulas2.calcularInscritosRangoEdad( RangoEdad.RANGO_4 ) );
        assertEquals( "El nmero de inscritos en el rango 6 (67 aos en adelante) no es el esperado.", 0, inscripcionCedulas2.calcularInscritosRangoEdad( RangoEdad.RANGO_5 ) );
    }

}