package uniandes.cupi2.inscripcionCedulas.test;

import uniandes.cupi2.inscripcionCedulas.excepciones.FormatoArchivoException;
import uniandes.cupi2.inscripcionCedulas.excepciones.PersistenciaException;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.InterpreteCSV;
import uniandes.cupi2.inscripcionCedulas.mundo.Departamento;
import uniandes.cupi2.inscripcionCedulas.mundo.InscripcionCedulas;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import junit.framework.TestCase;

/**
 * Clase para probar el interprete de archivos cvs
 * @author alvar-go
 *
 */
public class InterpreteCSVTest extends TestCase{

	/**
	 * Ruta con un archivo csv con formato correcto
	 */
	public final static String RUTA = "test/data/Puestos_de_votacion_test.csv";
	
	/**
	 * Ruta con un archivo csv con formato incorrecto
	 */
	public final static String RUTA_ERROR = "test/data/Puestos_de_votacion_test_error.csv";
	
	/**
	 * Clase donde se cargar la informacin del csv
	 */
	private InscripcionCedulas inscripcionCedulas;
	
	/**
	 * Escenario 1. sistema sin informacin
	 */
	public void setUp()
	{
		inscripcionCedulas = new InscripcionCedulas();
	}
	
    /**
     * Prueba 1: Se encarga de verificar el mtodo importarCSV de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * importarCSV<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay ciudadanos en el municipio y el archivo es correcto.<br>
     */
	public void testImportarCSV()
	{
		assertEquals("No debe existir informacin en el sistema", 0, inscripcionCedulas.darDepartamentos().size());
		
		try {
			InterpreteCSV.importarCSV(RUTA, inscripcionCedulas);
			assertEquals("Se deberan haber creado 2 departamentos", 2, inscripcionCedulas.darDepartamentos().size());
			
			assertNotNull("Se debera haber creado ANTIOQUIA", inscripcionCedulas.buscarDepartamento("ANTIOQUIA"));
			assertNotNull("Se debera haber creado PUTUMAYO", inscripcionCedulas.buscarDepartamento("PUTUMAYO"));
			assertNull("No se debera haber creado CUNDINAMARCA", inscripcionCedulas.buscarDepartamento("CUNDINAMARCA"));
			
			Departamento d = inscripcionCedulas.buscarDepartamento("ANTIOQUIA");
			assertEquals("ANTIOQUIA debera tener 1 municipio", 1, d.darMunicipios().size());
			
			assertNotNull("Debera existir MEDELLIN", d.buscarMunicipio("MEDELLIN"));
			assertNull("No debera existir BELLO", d.buscarMunicipio("BELLO"));
			
			Municipio m = d.buscarMunicipio("MEDELLIN");
			assertEquals("MEDELLIN debera tener 2 puestos", 2, m.darPuestosVotacion().size());
			
			assertNotNull("Debera existir INST.EDUC. LA CANDELARIA", m.buscarPuestoVotacion("INST.EDUC. LA CANDELARIA"));
			
			m = inscripcionCedulas.buscarDepartamento("PUTUMAYO").buscarMunicipio("MOCOA");
			assertNotNull("Debera existir MOCOA", m);
			assertEquals("MOCOA debera tener 3 puestos", 3,  m.darPuestosVotacion().size());
			
			assertNotNull("Debera existir CARCEL", m.buscarPuestoVotacion("CARCEL"));
			
			
		} 
		catch (PersistenciaException e) {
			fail("No debera generarse error al leer el archivo.");
		} 
		catch (FormatoArchivoException e) {
			fail("No debera generarse error con el formato del archivo.");
		}
	}
	
    /**
     * Prueba 2: Se encarga de verificar el mtodo importarCSV de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * importarCSV<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay ciudadanos en el municipio y el archivo es incorrecto.<br>
     */
	public void testImportarCSVErrorFormato()
	{
		assertEquals("No debe existir informacin en el sistema", 0, inscripcionCedulas.darDepartamentos().size());
		try 
		{
				InterpreteCSV.importarCSV(RUTA_ERROR, inscripcionCedulas);
				fail("No se debera completar el proceso, se debera lanzar un error de formato");
		} 
		catch (PersistenciaException e) 
		{
				fail("No debera haber error abriendo el archivo");
		} 
		catch (FormatoArchivoException e) 
		{
				//Est bien que entre ac
		}
			
	}
}
