/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.test;

import java.util.Calendar;
import java.util.Date;

import junit.framework.TestCase;
import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;

/**
 * Clase que verifica correcta implementacin de la clase Municipio.
 */
public class MunicipioTest extends TestCase
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase donde se harn las pruebas.
     */
    private Municipio municipio1;
    
    /**
     * Clase donde se harn las pruebas.
     */
    private Municipio municipio2;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Escenario 1: Construye un nuevo municipio vaco.
     * Escenario 2: Construye un nuevo municipio con puestos y ciudadanos.
     */
    public void setUp( )
    {
        municipio1 = new Municipio( "municipio" );
        
        municipio2 = new Municipio( "municipio" );

        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        calendar.set( anioActual - 30, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c2 = new Ciudadano( "456", "apellidos2", "nombre2", fechaNacimiento, "nacimiento2", Genero.FEMENINO, fechaExpedicion, "expedicion2" );

        calendar.set( anioActual - 20, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c3 = new Ciudadano( "789", "apellidos3", "nombre3", fechaNacimiento, "nacimiento3", Genero.MASCULINO, fechaExpedicion, "expedicion3" );

        municipio2.agregarPuestoVotacion( "puesto1", "dir1" );
        municipio2.agregarPuestoVotacion( "puesto2", "dir2" );
        municipio2.agregarPuestoVotacion( "puesto3", "dir3" );

        PuestoVotacion p1 = municipio2.buscarPuestoVotacion( "puesto1" );
        PuestoVotacion p2 = municipio2.buscarPuestoVotacion( "puesto2" );
        PuestoVotacion p3 = municipio2.buscarPuestoVotacion( "puesto3" );

        p1.inscribirCiudadano( c1 );
        p2.inscribirCiudadano( c2 );
        p3.inscribirCiudadano( c3 );
    }


    /**
     * Prueba 1: Se encarga de verificar el mtodo constructor de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * darNombre<br>
     * darNumInscritos<br>
     * darPuestosVotacion<br>
     * Municipio<br>
     * <b> Caso de prueba 1:</b><br>
     * Construye correctamente el municipio, cada uno de los valores corresponde al esperado.<br>
     */
    public void testMunicipio( )
    {

        assertEquals( "El nombre del municipio no es el esperado.", "municipio", municipio1.darNombre( ) );
        assertEquals( "El nmero de ciudadanos inscritos no es el esperado.", 0, municipio1.darNumInscritos( ) );
        assertNotNull( "No se ha inicializado la lista de puesto de votacin.", municipio1.darPuestosVotacion( ) );
    }

    /**
     * Prueba 2: Se encarga de verificar el mtodo agregarPuestoVotacion de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * agregarPuestoVotacion<br>
     * darPuestosVotacion<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay puestos de votacin en el municipio.<br>
     */
    public void testAgregarPuestoVotacion( )
    {

        PuestoVotacion p1 = new PuestoVotacion( "puesto1", "dir1" );
        municipio1.agregarPuestoVotacion( p1.darNombre( ), p1.darDireccion( ) );

        assertFalse( "No se ha agregado el puesto de votacin al municipio.", municipio1.darPuestosVotacion( ).isEmpty( ) );
        PuestoVotacion p = ( PuestoVotacion )municipio1.darPuestosVotacion( ).get( 0 );
        assertNotNull( "El puesto de votacin no fue agregado al municipio.", p );
    }

    /**
     * Prueba 3: Se encarga de verificar el mtodo buscarPuestoVotacion de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * buscarPuestoVotacion<br>
     * <b> Casos de prueba:</b><br>
     * 1. El municipio no est vaco y tiene puestos de votacin.<br>
     * 2. El municipio est vaco y no posee puestos de votacin.<br>
     */
    public void testBuscarPuestoVotacion( )
    {
        // 1
        PuestoVotacion puesto = municipio2.buscarPuestoVotacion( "puesto1" );
        assertNotNull( "El puesto de votacin debera ser encontrado.", puesto );

        // 2
        PuestoVotacion puesto2 = municipio2.buscarPuestoVotacion( "puesto" );
        assertNull( "No se debera encontrar el puesto de votacin.", puesto2 );

    }

    /**
     * Prueba 4: Se encarga de verificar el mtodo inscribirCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * inscribirCiudadano<br>
     * agregarPuestoVotacion<br>
     * existeCiudadano<br>
     * darCedula<br>
     * darApellidos<br>
     * darNombres<br>
     * darFechaNacimiento<br>
     * darLugarNacimiento<br>
     * darSexo<br>
     * darEdad<br>
     * darFechaExpedicion<br>
     * darLugarExpedicion<br>
     * darNumInscritos<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay ciudadanos en el municipio.<br>
     */
    public void testInscribirCiudadano( )
    {


        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        municipio1.agregarPuestoVotacion( "puesto1", "dir1" );
        municipio1.inscribirCiudadano( "puesto1", c1 );

        assertTrue( "El ciudadano no se encuentra registrado en el municipio.", municipio1.existeCiudadano( c1.darCedula( ) ) );
        assertEquals( "El nmero de ciudadanos inscritos no es el esperado.", 1, municipio1.darNumInscritos( ) );
        assertEquals( "El nmero de ciudadanos inscritos es el puesto puesto1 no es el esperado.", 1, municipio1.buscarPuestoVotacion( "puesto1" ).darNumInscritos( ) );
    }

    /**
     * Prueba 5: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El municipio est vaco por ende, no tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoError1( )
    {
        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = municipio1.existeCiudadano( c1.darCedula( ) );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 6: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El municipio no est vaco y no tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoError2( )
    {
        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "234", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = municipio2.existeCiudadano( c1.darCedula( ) );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 7: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El departamento no est vaco y tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoOK( )
    {
        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = municipio2.existeCiudadano( c1.darCedula( ) );
        assertTrue( "Se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 8: Se encarga de verificar el mtodo calcularNumeroInscritosPorGenero de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * calcularNumeroInscritosPorGenero<br>
     * <b> Casos de prueba:</b><br>
     * 1. El municipio tiene ciudadanos hombres inscritos.<br>
     * 2. El municipio tiene ciudadanos mujeres inscritas.<br>
     */
    public void testCalcularNumeroHombresInscritos( )
    {
        int hombres = municipio2.calcularNumeroInscritosPorGenero( Genero.MASCULINO);
        assertEquals( "El nmero de hombres no es el esperado.", 2, hombres );


        int mujeres = municipio2.calcularNumeroInscritosPorGenero(Genero.FEMENINO );
        assertEquals( "El nmero de mujeres no es el esperado.", 1, mujeres );
    }

    /**
     * Prueba 9: Se encarga de verificar el mtodo toString de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * toString<br>
     * <b> Casos de prueba:</b><br>
     * 1. El formato de texto es el correcto.<br>
     */
    public void testToString( )
    {
        assertEquals( "El formato del toString no es el esperado.", municipio1.darNombre( ), municipio1.toString( ) );
    }
}