/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id$
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n8_inscripcionCedulas
 * Autor: Equipo Cupi2 - 2015
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.inscripcionCedulas.test;

import java.util.Calendar;
import java.util.Date;

import junit.framework.TestCase;
import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;

/**
 * Clase que verifica correcta implementacin de la clase PuestoVotacion.
 */
public class PuestoVotacionTest extends TestCase
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Clase donde se harn las pruebas.
     */
    private PuestoVotacion puestoVotacion1;
    
    /**
     * Clase donde se harn las pruebas.
     */
    private PuestoVotacion puestoVotacion2;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Escenario 1: Construye un nuevo puesto de votacin vaco.
     * Escenario 2: Construye un nuevo puesto de votacin con ciudadanos inscritos.
     */
    public void setUp( )
    {
        puestoVotacion1 = new PuestoVotacion( "puesto", "dir" );
        
        puestoVotacion2 = new PuestoVotacion( "puesto", "dir" );

        Calendar calendar = Calendar.getInstance( );
        int anioActual = calendar.get( Calendar.YEAR );
        int mesActual = calendar.get( Calendar.MONTH );
        int diaActual = calendar.get( Calendar.DAY_OF_MONTH );

        calendar.set( anioActual - 50, mesActual, diaActual );
        Date fechaNacimiento = calendar.getTime( );
        calendar.set( anioActual, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );
        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        calendar.set( anioActual - 30, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c2 = new Ciudadano( "456", "apellidos2", "nombre2", fechaNacimiento, "nacimiento2", Genero.FEMENINO, fechaExpedicion, "expedicion2" );

        calendar.set( anioActual - 20, mesActual, diaActual );
        fechaNacimiento = calendar.getTime( );
        Ciudadano c3 = new Ciudadano( "789", "apellidos3", "nombre3", fechaNacimiento, "nacimiento3", Genero.MASCULINO, fechaExpedicion, "expedicion3" );

        puestoVotacion2.inscribirCiudadano( c1 );
        puestoVotacion2.inscribirCiudadano( c2 );
        puestoVotacion2.inscribirCiudadano( c3 );
    }

    /**
     * Prueba 1: Se encarga de verificar el mtodo constructor de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * darNombre<br>
     * darDireccion<br>
     * darInscritos<br>
     * darNumInscritos<br>
     * PuestoVotacion<br>
     * <b> Caso de prueba 1:</b><br>
     * Construye correctamente el puesto de votacin, cada uno de los valores corresponde al esperado.<br>
     */
    public void testPuestoVotacion( )
    {

        assertEquals( "El nombre del puesto de votacin no es el esperado.", "puesto", puestoVotacion1.darNombre( ) );
        assertEquals( "La direccin del puesto de votacin es la esperada.", "dir", puestoVotacion1.darDireccion( ) );
        assertNotNull( "La lista de ciudadanos inscritos no ha sido inicializada.", puestoVotacion1.darInscritos( ) );
        assertEquals( "El nmero de ciudadanos inscritos no es el esperado.", 0, puestoVotacion1.darNumInscritos( ) );
    }

    /**
     * Prueba 2: Se encarga de verificar el mtodo inscribirCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * inscribirCiudadano<br>
     * darCedula<br>
     * darApellidos<br>
     * darNombres<br>
     * darInscritos<br>
     * darFechaNacimiento<br>
     * darLugarNacimiento<br>
     * darSexo<br>
     * darEdad<br>
     * darFechaExpedicion<br>
     * darLugarExpedicion<br>
     * darNumInscritos<br>
     * <b> Caso de prueba 1:</b><br>
     * No hay ciudadanos en el puesto de votacin.<br>
     */
    public void testInscribirCiudadano( )
    {

        Calendar calendar = Calendar.getInstance( );
        calendar.set( 1900, 1, 1 );
        Date fechaNacimiento = calendar.getTime( );

        calendar.set( 1918, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );

        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );
        puestoVotacion1.inscribirCiudadano( c1 );

        assertFalse( "La lista de inscritos no debera estar vaca.", puestoVotacion1.darInscritos( ).isEmpty( ) );
        assertEquals( "El nmero de ciudadanos inscritos no es el esperado.", 1, puestoVotacion1.darNumInscritos( ) );

        Ciudadano c = puestoVotacion1.darInscritos( ).get( 0 );
        assertNotNull( "El ciudadano no fue agregado a la lista de inscritos.", c );
        assertTrue( "El ciudadano no fue agregado correctamente a la lista de inscritos.", c.darCedula( ).equals( c1.darCedula( ) ) );
    }

    /**
     * Prueba 3: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El puesto de votacin no est vaco y tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoOK( )
    {
        Calendar calendar = Calendar.getInstance( );
        calendar.set( 1900, 1, 1 );
        Date fechaNacimiento = calendar.getTime( );

        calendar.set( 1918, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );

        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = puestoVotacion2.existeCiudadano( c1.darCedula( ) );
        assertTrue( "Se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 4: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El puesto de votacin est vaco por ende, no tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoError1( )
    {
        Calendar calendar = Calendar.getInstance( );
        calendar.set( 1900, 1, 1 );
        Date fechaNacimiento = calendar.getTime( );

        calendar.set( 1918, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );

        Ciudadano c1 = new Ciudadano( "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = puestoVotacion1.existeCiudadano( c1.darCedula( ) );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 5: Se encarga de verificar el mtodo existeCiudadano de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * existeCiudadano<br>
     * darCedula<br>
     * <b> Casos de prueba:</b><br>
     * 1. El puesto de votacin no est vaco y no tiene ciudadanos.<br>
     */
    public void testExisteCiudadanoError2( )
    {
        Calendar calendar = Calendar.getInstance( );
        calendar.set( 1900, 1, 1 );
        Date fechaNacimiento = calendar.getTime( );

        calendar.set( 1918, 2, 1 );
        Date fechaExpedicion = calendar.getTime( );

        Ciudadano c1 = new Ciudadano( "234", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1" );

        boolean existe = puestoVotacion2.existeCiudadano( c1.darCedula( ) );
        assertFalse( "No se debera encontrar al ciudadano.", existe );
    }

    /**
     * Prueba 6: Se encarga de verificar el mtodo calcularInscritosPorGenero de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * calcularInscritosPorGenero<br>
     * <b> Casos de prueba:</b><br>
     * 1. El puesto de votacin tiene ciudadanos hombres inscritos.<br>
     * 2. El puesto de votacin tiene ciudadanos mujeres inscritas.<br>
     */
    public void testCalcularNumeroHombresInscritos( )
    {

        int hombres = puestoVotacion2.calcularInscritosPorGenero(Genero.MASCULINO);
        assertEquals( "El nmero de hombres no es el esperado.", 2, hombres );
        
        int mujeres = puestoVotacion2.calcularInscritosPorGenero(Genero.FEMENINO);
        assertEquals( "El nmero de mujeres no es el esperado.", 1, mujeres );
    }


    /**
     * Prueba 7: Se encarga de verificar el mtodo toString de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * toString<br>
     * <b> Casos de prueba:</b><br>
     * 1. El formato de texto es el correcto.<br>
     */
    public void testToString( )
    {
        assertEquals( "El formato del toString no es el esperado.", puestoVotacion1.darNombre( ), puestoVotacion1.toString( ) );
    }
}
