package uniandes.cupi2.inscripcionCedulas.test;

import java.io.File;
import java.util.Calendar;
import java.util.Date;

import junit.framework.TestCase;

import uniandes.cupi2.inscripcionCedulas.excepciones.PersistenciaException;
import uniandes.cupi2.inscripcionCedulas.manejoArchivos.Serializador;
import uniandes.cupi2.inscripcionCedulas.mundo.Ciudadano;
import uniandes.cupi2.inscripcionCedulas.mundo.Departamento;
import uniandes.cupi2.inscripcionCedulas.mundo.Genero;
import uniandes.cupi2.inscripcionCedulas.mundo.InscripcionCedulas;
import uniandes.cupi2.inscripcionCedulas.mundo.Municipio;
import uniandes.cupi2.inscripcionCedulas.mundo.PuestoVotacion;

/**
 * Clase que prueba el correcto funcionamiento de la clase Serializador
 * @author alvar-go
 *
 */
public class SerializadorTest extends TestCase{

	/**
	 * Ruta para guardar el archivo
	 */
	public final static String RUTA = "test/data/inscripcionTest.data";
	
	// -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------
    
    /**
     * Clase donde se harn las pruebas.
     */
    private InscripcionCedulas inscripcionCedulas1;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Escenario 1: Construye un nuevo sistema InscripcionCedulas vaco.  
     * Escenario 2: Construye un nuevo sistema InscripcionCedulas con departamentos, municipios, puesto de votacin y ciudadanos.
     */
    public void setUp( )
    {
        
        inscripcionCedulas1 = new InscripcionCedulas();
       
        Departamento d1 = new Departamento("departamento1");
        Departamento d2 = new Departamento("departamento2");
        
        d1.agregarMunicipio( "municipio1" );
        d2.agregarMunicipio( "municipio2" );
        d2.agregarMunicipio( "municipio3" );

        Municipio m1 = d1.buscarMunicipio( "municipio1" );
        Municipio m2 = d2.buscarMunicipio( "municipio2" );
        Municipio m3 = d2.buscarMunicipio( "municipio3" );

        m1.agregarPuestoVotacion( "puesto1", "dir1" );
        m2.agregarPuestoVotacion( "puesto2", "dir2" );
        m3.agregarPuestoVotacion( "puesto3", "dir3" );
        
        PuestoVotacion p1 = m1.buscarPuestoVotacion("puesto1");
        PuestoVotacion p2 = m2.buscarPuestoVotacion("puesto2");
        PuestoVotacion p3 = m3.buscarPuestoVotacion("puesto3");
        
        inscripcionCedulas1.agregarDepartamento(d1);
        inscripcionCedulas1.agregarDepartamento(d2);
        
        try {
	    	 Calendar calendar = Calendar.getInstance( );
	         int anioActual = calendar.get( Calendar.YEAR );
	         int mesActual = calendar.get( Calendar.MONTH );
	         int diaActual = calendar.get( Calendar.DAY_OF_MONTH );
	
	         calendar.set( anioActual - 50, mesActual, diaActual );
	         Date fechaNacimiento = calendar.getTime( );
	         calendar.set( anioActual, 2, 1 );
	         Date fechaExpedicion = calendar.getTime( );
        	inscripcionCedulas1.inscribirCiudadano(p1, "123", "apellidos1", "nombre1", fechaNacimiento, "nacimiento1", Genero.MASCULINO, fechaExpedicion, "expedicion1");
        	
        	calendar.set( anioActual - 30, mesActual, diaActual );
            fechaNacimiento = calendar.getTime( );
        	inscripcionCedulas1.inscribirCiudadano(p2, "456", "apellidos2", "nombre2", fechaNacimiento, "nacimiento2", Genero.FEMENINO, fechaExpedicion, "expedicion2");
        	
        	calendar.set( anioActual - 20, mesActual, diaActual );
            fechaNacimiento = calendar.getTime( );
        	inscripcionCedulas1.inscribirCiudadano(p3, "789", "apellidos3", "nombre3", fechaNacimiento, "nacimiento3", Genero.MASCULINO, fechaExpedicion, "expedicion3");
			
		} catch (Exception e) {
			e.printStackTrace();
			fail("No debera haber problema agreando ciudadanos");
		}
    }
    
    /**
     * Prueba 1: Se encarga de verificar los mtodos cargar y guardar de la clase.<br>
     * <b> Mtodos a probar: </b> <br>
     * guardar<br>
     * cargar<br>
     * <b> Caso de prueba 1:</b><br>
     * 1. Serializa correctamente un escenario con departamentos, municipios y puestos.<br>
     */
    public void testGuardarCargarTest()
    {
    	try 
    	{
			Serializador.guardar(RUTA, inscripcionCedulas1);
			File f = new File(RUTA);
			assertTrue("Se debera haber creado el archivo", f.exists());
		} 
    	catch (PersistenciaException e) 
    	{
			fail("No deberan ocurrir problemas al guardar: " + e.getMessage());
		}
    	
    	InscripcionCedulas inc = new InscripcionCedulas();
    	try 
    	{
			inc = Serializador.cargar(RUTA, inc);
			assertEquals("La informacin cargada no es correcta", inscripcionCedulas1.darNumInscritos(), inc.darNumInscritos());
			assertEquals("La informacin cargada no es correcta", inscripcionCedulas1.darDepartamentos().size(), inc.darDepartamentos().size());
			
			for (Departamento departamento : inscripcionCedulas1.darDepartamentos()) 
			{
				Departamento d  = inc.buscarDepartamento(departamento.darNombre());
				assertNotNull("Debera existir el departamento", d);
				for (Municipio municipio : departamento.darMunicipios()) 
				{
					Municipio m = d.buscarMunicipio(municipio.darNombre());
					assertNotNull("Debera existir el municipio", m);
					for (PuestoVotacion puestoVotacion : municipio.darPuestosVotacion()) 
					{
						PuestoVotacion p = m.buscarPuestoVotacion(puestoVotacion.darNombre());
						assertNotNull("Debera existir el puesto de votacin", p);
						for (Ciudadano ciudadano : puestoVotacion.darInscritos()) 
						{
							assertTrue("Debera existir el ciudadano", p.existeCiudadano(ciudadano.darCedula()));
						}
					}
					
				}
			}
			File f = new File(RUTA);
			f.delete();
		} 
    	catch (PersistenciaException e) {
			fail("No deberan ocurrir problemas al cargar: " + e.getMessage());
		}
    }
}
